@@ -1,16 +1,17 @@
-/*  $Revision: 1.2 $
+/*  $Revision: 1.7 $
 **
 **  Get a file list from an NNTP server.
 */
-#include "configdata.h"
 #include <stdio.h>
-#include <errno.h>
 #include <sys/types.h>
+#include "configdata.h"
 #include "clibrary.h"
+#include <errno.h>
 #include "libinn.h"
 #include "qio.h"
 #include "paths.h"
 #include "macros.h"
+#include <syslog.h>  
 
 
 /*
@@ -19,7 +20,7 @@
 STATIC NORETURN
 Usage()
 {
-    (void)fprintf(stderr, "Usage: getlist [-h host] [type [pat [groups]]\n");
+    (void)fprintf(stderr, "Usage: getlist [-p port] [-h host] [type [pat [groups]]\n");
     exit(1);
 }
 
@@ -41,15 +42,22 @@ main(ac, av)
     char	*p;
     char	*pattern;
     char	buff[512 + 1];
+    int		port;
     int		i;
 
+    /* First thing, set up logging and our identity. */
+    openlog("getlist", L_OPENLOG_FLAGS | LOG_PID, LOG_INN_PROG);     
+
+    if (ReadInnConf() < 0) exit(1);
+
     /* Set defaults. */
     host = NULL;
     pattern = NULL;
     types = NULL;
+    port = NNTP_PORT;
 
     /* Parse JCL. */
-    while ((i = getopt(ac, av, "h:")) != EOF)
+    while ((i = getopt(ac, av, "h:p:")) != EOF)
 	switch (i) {
 	default:
 	    Usage();
@@ -57,6 +65,13 @@ main(ac, av)
 	case 'h':
 	    host = optarg;
 	    break;
+	case 'p':
+	    port = atoi(optarg);
+	    if (port <= 0) {
+	       (void)fprintf(stderr, "Illegal value for -p option\n");
+	       	exit(1);
+	    }
+	    break;
 	}
     ac -= optind;
     av += optind;
@@ -67,11 +82,12 @@ main(ac, av)
 	Usage();
 	/* NOTREACHED */
     case 0:
-	break;
     case 1:
-	pattern = av[1];
 	break;
     case 2:
+	pattern = av[1];
+	break;
+    case 3:
 	pattern = av[1];
 	types = av[2];
 	break;
@@ -79,20 +95,20 @@ main(ac, av)
     if (av[0] == NULL)
 	list = "active";
     else {
-	if (pattern != NULL || types != NULL)
-	    Usage();
 	list = av[0];
+	if (!EQ(list, "active") && pattern != NULL)
+	    Usage();
     }
 
     /* Open a connection to the server. */
     if (host == NULL
-     && (host = GetConfigValue(_CONF_SERVER)) == NULL) {
+     && (host = innconf->server) == NULL) {
 	(void)fprintf(stderr, "Can't get server name, %s\n", strerror(errno));
 	exit(1);
     }
     buff[0] = '\0';
-    if (NNTPconnect(host, &FromServer, &ToServer, buff) < 0) {
-	(void)fprintf(stderr, "Can't connect to server, %s.\n",
+    if (NNTPconnect(host, port, &FromServer, &ToServer, buff) < 0) {
+	(void)fprintf(stderr, "Can't connect to server, %s\n",
 		buff[0] ? buff : strerror(errno));
 	exit(1);
     }
@@ -107,7 +123,7 @@ main(ac, av)
     }
 
     /* Set up to read it quickly. */
-    if ((qp = QIOfdopen((int)fileno(active), QIO_BUFFER)) == NULL) {
+    if ((qp = QIOfdopen((int)fileno(active))) == NULL) {
 	(void)fprintf(stderr, "Can't read temp file, %s\n", strerror(errno));
 	(void)fclose(FromServer);
 	(void)fclose(ToServer);
@@ -176,8 +192,10 @@ main(ac, av)
 
     /* All done. */
     CAclose();
-    (void)fclose(FromServer);
+    (void)fprintf(ToServer, "quit\r\n");
     (void)fclose(ToServer);
-    exit(1);
+    (void)fgets(buff, sizeof buff, FromServer);
+    (void)fclose(FromServer);
+    exit(i);
     /* NOTREACHED */
 }
