@@ -1,5 +1,5 @@
 /*
- * $Revision: 1.6 $
+ * $Revision: 1.6.2.3 $
  */
 
 
@@ -13,10 +13,10 @@
 **  Try to avoid changing these.
 */
 #define RELEASE "2"
-#define PATCHLEVEL "1"
-#define DATE "24-Jul-1998"
+#define PATCHLEVEL "2"
+#define DATE "21-Jan-1999"
 #define RELEASE_C 2
-#define PATCHLEVEL_C 1
+#define PATCHLEVEL_C 2
 #define SUB_PATCHLEVEL_C 0
 
 
