@@ -1,4 +1,4 @@
-/*  $Id: chan.c,v 1.28 1998/07/02 02:16:58 kondou Exp $
+/*  $Id: chan.c,v 1.32.2.1 1998/11/11 22:56:12 kondou Exp $
 **
 **  I/O channel (and buffer) processing.
 */
@@ -85,6 +85,24 @@ void BUFFswap(BUFFER *b1, BUFFER *b2)
     *b2 = b;
 }
 
+/*
+**  Trim '\r' from buffer.
+*/
+void BUFFtrimcr(BUFFER *bp)
+{
+    char *p, *q;
+    int trimmed = 0;
+
+    for (p = q = bp->Data ; p < bp->Data + bp->Left ; p++) {
+	if (*p == '\r' && p+1 < bp->Data + bp->Left && p[1] == '\n') {
+	    trimmed++;
+	    continue;
+	}
+	*q++ = *p;
+    }
+    bp->Left -= trimmed;
+}
+
 
 /*
 **  Initialize all the I/O channels.
@@ -161,12 +179,19 @@ CHANNEL *CHANcreate(int fd, CHANNELTYPE Type, CHANNELSTATE State,
     cp->Rest=0;
     cp->SaveUsed=0;
     cp->Lastch=0;
+    cp->MaxCnx=0;
+    cp->ActiveCnx=0;
     HashClear(&cp->CurrentMessageIDHash);
     memset(cp->PrecommitWIP, '\0', sizeof(cp->PrecommitWIP));
     cp->PrecommitiCachenext=0;
 
     /* Make the descriptor close-on-exec and non-blocking. */
     CloseOnExec(fd, TRUE);
+#if !defined(HPUX)
+    /* Stupid HPUX 11.0 has a broken listen/accept where setting the
+     * listensocket to nonblocking prevents you from successfully setting the
+     * socket returned by accept(2) back to blocking mode, no matter what,
+     * resulting in all kinds of funny behaviour, data loss, etc. etc.  */
     if (SetNonBlocking(fd, TRUE) < 0
 #if	defined(ENOTSOCK)
 	    && errno != ENOTSOCK
@@ -176,6 +201,7 @@ CHANNEL *CHANcreate(int fd, CHANNELTYPE Type, CHANNELSTATE State,
 #endif	/* defined(ENOTTY) */
     )
 	syslog(L_ERROR, "%s cant nonblock %d %m", LogName, fd);
+#endif
 
     /* Note control channel, for efficiency. */
     if (Type == CTcontrol) {
@@ -250,6 +276,24 @@ void CHANclose(CHANNEL *cp, char *name)
 	    DISPOSE(cp->Argument);
 	if (cp->fd >= 0 && close(cp->fd) < 0)
 	    syslog(L_ERROR, "%s cant close %s %m", LogName, name);
+ 
+	if (cp->MaxCnx > 0 && cp->Type == CTnntp) {
+	    int tfd;
+	    CHANNEL *tempchan;
+
+	    cp->fd = -1;
+	    for(tfd = 0; tfd < CHANlastfd; tfd++) {
+		tempchan = &CHANtable[tfd];
+		if(tempchan->fd > 0 &&
+		    cp->Address.s_addr == tempchan->Address.s_addr) {
+		    if(tempchan->ActiveCnx == 0) {
+			tempchan->ActiveCnx = cp->ActiveCnx;
+			RCHANadd(tempchan);
+		        break;
+		    }
+		}
+	    }
+	}
     }
 
     /* Mark it unused. */
@@ -523,9 +567,11 @@ int CHANreadtext(CHANNEL *cp)
     BUFFER	        *bp;
     char		*p;
     int			oerrno;
+    int			maxbyte;
 
     /* Grow buffer if we're getting close to current limit. */
     bp = &cp->In;
+    bp->Left = bp->Size - bp->Used;
     if (bp->Left <= LOW_WATER) {
 	i = GROW_AMOUNT(bp->Size);
 	bp->Size += i;
@@ -549,9 +595,11 @@ int CHANreadtext(CHANNEL *cp)
      * data, or less.  BUFSIZ is often about 1 kilobyte, and is
      * attractive for other reasons, so let's use that as our size limit.
      */
-    bp->Left = bp->Size - bp->Used;
-    i = read(cp->fd, &bp->Data[bp->Used],
-	      (bp->Left - 1 > BUFSIZ ? BUFSIZ : bp->Left - 1));
+    /*
+     * Reduce the read size only if we are reading commands.
+     */
+    maxbyte = (cp->State != CSgetcmd || bp->Left < BUFSIZ) ? bp->Left : BUFSIZ;
+    i = read(cp->fd, &bp->Data[bp->Used], maxbyte-1);
     if (i < 0) {
 #ifdef POLL_BUG
     /* return of -2 indicates EAGAIN, for SUNOS5.4 poll() bug workaround */
@@ -859,6 +907,8 @@ void CHANreadloop(void)
                             fd = 0;
                         else
                             fd++;
+			cp->ActiveCnx = 0;
+			RCHANremove(cp);
                     }
                     continue;
                 }
