@@ -1,4 +1,4 @@
-/*  $Id: innd.c,v 1.38 1998/05/25 22:40:20 kondou Exp $
+/*  $Id: innd.c,v 1.40 1998/09/05 15:04:28 kondou Exp $
 **
 **  Variable definitions, miscellany, and main().
 */
@@ -639,8 +639,8 @@ int main(int ac, char *av[])
 	    default:
 		Usage();
 		/* NOTREACHED */
-	    case 'n':	innconf->allowreaders = TRUE;	break;
-	    case 'y':	innconf->allowreaders = FALSE;	break;
+	    case 'n':	innconf->readerswhenstopped = TRUE;	break;
+	    case 'y':	innconf->readerswhenstopped = FALSE;	break;
 	    }
 	    break;
 	case 'o':
@@ -688,7 +688,7 @@ int main(int ac, char *av[])
     ac -= optind;
     if (ac != 0)
 	Usage();
-    if (ModeReason && innconf->allowreaders)
+    if (ModeReason && innconf->readerswhenstopped)
 	NNRPReason = COPY(ModeReason);
 
     openlog(path, logflags, LOG_INN_SERVER);
@@ -927,6 +927,7 @@ int main(int ac, char *av[])
     p = COPY(cpcatpath(innconf->pathfilter, _PATH_PERL_STARTUP_INND));
     PERLsetup(p, cpcatpath(innconf->pathfilter, _PATH_PERL_FILTER_INND),
 				"filter_art");
+    PERLinndAPIinit();
     PerlFilter (TRUE) ;
     DISPOSE(p);
 #endif /* defined(DO_PERL) */
