@@ -1,4 +1,4 @@
-/*  $Id: nc.c,v 1.35 1998/07/16 23:04:21 kondou Exp $
+/*  $Id: nc.c,v 1.37.2.4 1999/01/17 23:15:13 kondou Exp $
 **
 **  Routines for the NNTP channel.  Other channels get the descriptors which
 **  we turn into NNTP channels, and over which we speak NNTP.
@@ -471,7 +471,7 @@ NCihave(CHANNEL *cp)
 	    else cp->Sendid.Size = MAXHEADERSIZE;
 	    cp->Sendid.Data = NEW(char, cp->Sendid.Size);
 	}
-	(void)sprintf(cp->Sendid.Data, "%d %s", NNTP_ERR_GOTID_VAL, p);
+	(void)sprintf(cp->Sendid.Data, "%d %s", NNTP_HAVEIT_VAL, perlrc);
 	NCwritereply(cp, cp->Sendid.Data);
 	return;
     }
@@ -484,7 +484,12 @@ NCihave(CHANNEL *cp)
     }
     else if (WIPinprogress(p, cp, FALSE)) {
 	cp->Ihave_Deferred++;
-	NCwritereply(cp, NNTP_RESENDIT_LATER);
+	if (cp->NoResendId) {
+	    cp->Refused++;
+	    NCwritereply(cp, NNTP_HAVEIT);
+	} else {
+	    NCwritereply(cp, NNTP_RESENDIT_LATER);
+	}
     }
     else {
 	cp->Ihave_SendIt++;
@@ -560,11 +565,19 @@ NClist(CHANNEL *cp)
     if (caseEQ(p, "newsgroups")) {
 	trash = p = ReadInFile(cpcatpath(innconf->pathdb, _PATH_NEWSGROUPS),
 			(struct stat *)NULL);
+	if (p == NULL) {
+	    NCwritereply(cp, NCdot);
+	    return;
+	}
 	end = p + strlen(p);
     }
     else if (caseEQ(p, "active.times")) {
 	trash = p = ReadInFile(cpcatpath(innconf->pathdb, _PATH_ACTIVETIMES),
 			(struct stat *)NULL);
+	if (p == NULL) {
+	    NCwritereply(cp, NCdot);
+	    return;
+	}
 	end = p + strlen(p);
     }
     else if (*p == '\0' || (caseEQ(p, "active"))) {
@@ -602,7 +615,7 @@ NCmode(CHANNEL *cp)
     for (p = cp->In.Data + STRLEN("mode"); ISWHITE(*p); p++)
 	continue;
 
-    if (caseEQ(p, "reader"))
+    if (caseEQ(p, "reader") && !innconf->noreader)
 	h = HOnnrpd;
     else if (caseEQ(p, "stream") &&
              (!StreamingOff && cp->Streaming)) {
@@ -643,6 +656,11 @@ NCxpath(CHANNEL *cp)
     char		*p;
     int			i;
 
+    if (innconf->storageapi) {
+	/* not available for storageapi */
+	NCwritereply(cp, NNTP_BAD_COMMAND);
+	return;
+    }
     /* Nip off the Message-ID. */
     for (p = cp->In.Data + STRLEN("xpath"); ISWHITE(*p); p++)
 	continue;
@@ -898,6 +916,7 @@ STATIC FUNCTYPE NCproc(CHANNEL *cp)
 	    if (Mode == OMpaused) { /* defer processing while paused */
 		cp->Rest = 0;
 		bp->Used = cp->SaveUsed;
+		RCHANremove(cp); /* don't bother trying to read more for now */
 		SCHANadd(cp, (time_t)(Now.time + innconf->pauseretrytime),
 		    (POINTER)&Mode, NCproc, (POINTER)NULL);
 		return;
