@@ -1,4 +1,4 @@
-/*  $Id: newsfeeds.c,v 1.18 1998/04/21 04:29:57 scrappy Exp $
+/*  $Id: newsfeeds.c,v 1.18.4.2 1999/01/07 10:19:53 kondou Exp $
 **
 **  Routines for the in-core data structures for the newsfeeds file.
 */
@@ -320,7 +320,9 @@ STRING SITEparseone(char *Entry, SITE *sp, char *subbed, char *poison)
 		}
 	    break;
 	case 'O':
-	    sp->Originator = COPY(++p);
+	    if (*++p == '\0')
+		return "missing originator name for O param in field 3";
+	    sp->Originator = COPY(p);
 	    break;
         case 'P':
             if (*++p && CTYPE(isdigit, *p))
@@ -559,6 +561,7 @@ SITEparsefile(StartSite)
     char		*poison;
     STRING		error;
     int			errors;
+    int			setuperrors;
 
     /* Free old sites info. */
     if (Sites) {
@@ -581,7 +584,7 @@ SITEparsefile(StartSite)
     poison = NEW(char, nGroups);
 
     ME.Prev = 0; /* Used as a flag to ensure exactly one ME entry */
-    for (sp = Sites, errors = 0, i = 0; i < nSites; i++) {
+    for (sp = Sites, errors = 0, setuperrors = 0, i = 0; i < nSites; i++) {
 	p = strings[i];
 	if (p[0] == 'M' && p[1] == 'E' &&
             ((p[2] == NF_FIELD_SEP) || (p[2] == NF_SUBFIELD_SEP))) {
@@ -602,7 +605,7 @@ SITEparsefile(StartSite)
 	}
 	if (StartSite && !SITEsetup(sp)) {
 	    syslog(L_FATAL, "%s cant setup %m", sp->Name);
-	    errors++;
+	    setuperrors++;
 	    continue;
 	}
 	sp->Working = TRUE;
@@ -613,8 +616,11 @@ SITEparsefile(StartSite)
 	errors++;
     }
 
-    if (errors) {
-	syslog(L_FATAL, "%s syntax_error %s", LogName, SITEfeedspath);
+    if (errors && setuperrors) {
+	if (errors)
+	    syslog(L_FATAL, "%s syntax_error %s", LogName, SITEfeedspath);
+	if (setuperrors)
+	    syslog(L_FATAL, "%s setup_error %s", LogName, SITEfeedspath);
 	JustCleanup();
 	exit(1);
     }
