@@ -1,4 +1,4 @@
-/*  $Id: rc.c,v 1.29 1998/07/02 02:17:23 kondou Exp $
+/*  $Id: rc.c,v 1.31.2.2 1999/01/19 03:14:56 kondou Exp $
 **
 **  Routines for the remote connect channel.  Create an Internet stream socket
 **  that processes connect to.  If the incoming site is not one of our feeds,
@@ -469,7 +469,8 @@ RCreader(CHANNEL *cp)
 	    break;
 	}
 
-    /* If not a server, and not allowing anyone, hand him off. */
+    /* If not a server, and not allowing anyone, hand him off unless
+       not spawning nnrpd in which case we return an error. */
     if ((i >= 0) && !rp->Skip) {
 	new = NCcreate(fd, rp->Password[0] != '\0', FALSE);
         new->Streaming = rp->Streaming;
@@ -478,11 +479,23 @@ RCreader(CHANNEL *cp)
         new->MaxCnx = rp->MaxCnx;
     } else if (AnyIncoming && !rp->Skip) {
 	new = NCcreate(fd, FALSE, FALSE);
-    } else {
+    } else if (!innconf->noreader) {
 	RChandoff(fd, HOnntpd);
 	if (close(fd) < 0)
 	    syslog(L_ERROR, "%s cant close %d %m", LogName, fd);
 	return;
+    } else {
+	reject_val = NNTP_ACCESS_VAL;
+	reject_message = NNTP_ACCESS;
+        new = CHANcreate(fd, CTreject, CSwritegoodbye, RCrejectreader,
+            RCrejectwritedone);
+        new->Address.s_addr = remote.sin_addr.s_addr;
+        new->Rejected = reject_val;
+        RCHANremove(new);
+        WCHANset(new, reject_message, (int)strlen(reject_message));
+        WCHANappend(new, RCterm, STRLEN(RCterm));
+        WCHANadd(new);
+        return;
     }
 
     new->Address.s_addr = remote.sin_addr.s_addr;
@@ -801,11 +814,10 @@ RCreadfile (REMOTEHOST_DATA **data, REMOTEHOST **list, int *count,
 	if (peer_params.Label != NULL) {
 	  RCadddata(data, &infocount, K_END_PEER, T_STRING, NULL);
 
-	  /* Hostname */
-	  if (peer_params.Name == NULL) {
-	    syslog(L_ERROR, HOST_NEEDED, LogName, filename, linecount);
-	    break;
-	  }
+	  /* Hostname defaults to label if not given */
+	  if (peer_params.Name == NULL)
+            peer_params.Name = COPY(peer_params.Label);
+
 	  for(r = q = RCCommaSplit(COPY(peer_params.Name)); *q != NULL; q++) {
 	    (*count)++;
 
@@ -835,6 +847,8 @@ RCreadfile (REMOTEHOST_DATA **data, REMOTEHOST **list, int *count,
 	    /* Host specified as a text name ? */
 	    if ((hp = gethostbyname(*q)) == NULL) {
 	      syslog(L_ERROR, "%s cant gethostbyname %s %m", LogName, *q);
+	      /* decrement *count, since we never got to add this record. */
+	      (*count)--;
 	      continue;
 	    }
 
@@ -1202,12 +1216,12 @@ RCwritelist(char *filename)
     /* Write a standard header.. */
 
     /* Find the filename */
-    p = cpcatpath(innconf->pathetc, _PATH_INNDHOSTS);
+    p = COPY(cpcatpath(innconf->pathetc, _PATH_INNDHOSTS));
     for (r = q = p; *p; p++)
         if (*p == '/')
 	   q = p + 1;
 
-    fprintf (F, "##  $Revision: 1.29 $\n");
+    fprintf (F, "##  $Revision: 1.31.2.2 $\n");
     fprintf (F, "##  %s - names and addresses that feed us news\n", q);
     DISPOSE(r);
     fprintf (F, "##\n\n");
