@@ -1,5 +1,5 @@
 
-/* $Id: status.c,v 1.11 1998/06/09 10:53:49 scrappy Exp $ */
+/* $Id: status.c,v 1.12 1998/08/24 16:29:51 kondou Exp $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -16,7 +16,7 @@
 #define MAX_PEER     100
 #define MIN_REFRESH   60  /* 1 min */
 #define HTML_STATUS
-#define STATUS_FILE  "inn.status" /* will be in pathhttp */
+#define STATUS_FILE  "inn.status" /* will be in pathlog/pathhttp */
 
 typedef struct _STATUS {
     char           name[SMBUF];
@@ -112,7 +112,11 @@ static void STATUSsummary(void)
   extern int        PerlFilterActive;
 #endif /* defined(DO_PERL) */
  
+#if defined(HTML_STATUS)
   sprintf (TempString, "%s/%s", innconf->pathhttp, STATUS_FILE);
+#else
+  sprintf (TempString, "%s/%s", innconf->pathlog, STATUS_FILE);
+#endif
   if ((F = Fopen(TempString, "w", TEMPORARYOPEN)) == NULL)
     return;
 
