@@ -1,5 +1,5 @@
 /*
- * $Id: startinnfeed.c,v 1.7 1998/07/21 14:43:40 kondou Exp $
+ * $Id: startinnfeed.c,v 1.8 1998/07/30 23:36:01 kondou Exp $
  *
  * Start innfeed and pass it all the arguments given. Sets up process
  * limits for innfeed.
@@ -26,7 +26,7 @@
 #include "clibrary.h"
 #include "libinn.h"
 
-void
+int
 main(int ac, char **av, char **ep)
 {
   struct passwd *pwd;
