@@ -1,4 +1,4 @@
-/*  $Revision: 1.45 $
+/*  $Revision: 1.57.2.5 $
 **
 **  Article-related routines.
 */
@@ -36,15 +36,6 @@ typedef struct _SENDDATA {
     STRING	Item;
 } SENDDATA;
 
-typedef struct _OVERDATA {
-    int		index;
-    char	*addr;
-    long	size;
-    FILE	*fp;
-    int		offset;
-} OVERDATA;
-
-
 /*
 **  Information about the schema of the news overview files.
 */
@@ -82,7 +73,6 @@ STATIC SENDDATA		SENDhead = {
 
 extern unsigned int RARTtable[];
 extern int RARTcount;
-extern int RARTenable;
 
 /*
 **  Overview state information.
@@ -97,6 +87,10 @@ STATIC char *		OVERmempos;		/* Current position in mmaped overview file */
 STATIC struct iovec	iov[IOV_MAX];
 STATIC int		queued_iov = 0;
 
+/* Prototypes */
+STATIC BOOL IsCancelled(int artnum, BOOL checkart);
+STATIC BOOL IsCancelledByIndex(int i, int artnum);
+
 BOOL PushIOv(void) {
     fflush(stdout);
     if (writev(STDOUT_FILENO, iov, queued_iov) <= 0) {
@@ -205,7 +199,7 @@ void ARTclose(void)
 	if (ARThandle) {
 	    SMfreearticle(ARThandle);
 	} else {
-#if defined(MC_ADVISE) && defined(MADV_DONTNEED) && !defined(_nec_ews)
+#if defined(MADV_DONTNEED) && defined(HAVE_MADVISE)
 	    madvise(ARTmem, ARTlen, MADV_DONTNEED);
 #endif
 	    munmap(ARTmem, ARTlen);
@@ -229,6 +223,9 @@ STATIC BOOL ARTinstore(int i)
     ARTHANDLE *art;
     struct timeval	stv, etv;
 
+    if (IsCancelledByIndex(i,0))
+        return FALSE;
+
     if (ARTnumbers[i].Tokenretrieved)
 	token = ARTnumbers[i].Token;
     else {
@@ -237,12 +234,19 @@ STATIC BOOL ARTinstore(int i)
 	   return FALSE;
 	}
 	UnpackOverIndex(*(ARTnumbers[i].Index), &index);
-	ARTnumbers[i].Tokenretrieved = TRUE;
-	if ((tokentext = HISgetent(&index.hash, FALSE, NULL)) == (char *)NULL) {
-	   ARTnumbers[i].Token.cancelled = TRUE;
-	   return FALSE;
+	if (HashEmpty(index.hash)) 
+	    return FALSE;
+	if (ARTnumbers[i].Offset >= 0) {
+	    tokentext = HISgetent(&index.hash, TRUE, &ARTnumbers[i].Offset);
+	} else {
+	    tokentext = HISgetent(&index.hash, FALSE, NULL);
+	}
+	if (tokentext == (char *)NULL) {
+	    ARTnumbers[i].Token.cancelled = TRUE;
+	    return FALSE;
 	}
 	token = ARTnumbers[i].Token = TextToToken(tokentext);
+	ARTnumbers[i].Tokenretrieved = TRUE;
 	if (   token.type == TOKEN_EMPTY
 	    || token.index == OVER_NONE
 	    || token.cancelled )
@@ -259,7 +263,8 @@ STATIC BOOL ARTinstore(int i)
     if (art) {
 	SMfreearticle(art);
 	return TRUE;
-    } else return FALSE;
+    } 
+    return FALSE;
 }
 
 /*
@@ -277,21 +282,62 @@ STATIC int ARTfind(ARTNUM i, BOOL needcheck)
 	return -1;
 
     top = &ARTnumbers[ARTsize - 1];
-    if (ARTcache && (++ARTcache <= top) && (ARTcache->ArtNum <= i)) {
-	if (ARTcache->ArtNum == i) {
+    if (ARTcache) {
+        /* They're finding the same article again. */
+        if (ARTcache->ArtNum == i) {
 	    if (needcheck) {
 		return ARTinstore(ARTcache - ARTnumbers)
 		    ? (ARTcache - ARTnumbers) : -1;
 	    }
-	    return ARTcache - ARTnumbers;
-	}
-	bottom = ARTcache;
-    }
-    else {
-	ARTcache = NULL;
-	bottom = ARTnumbers;
+            else
+                return ARTcache - ARTnumbers;
+        }
+        else if ((++ARTcache <= top) && (ARTcache->ArtNum <= i)) {
+            /* Next article. */
+            if (ARTcache->ArtNum == i) {
+                if (needcheck) {
+                    return ARTinstore(ARTcache - ARTnumbers)
+                        ? (ARTcache - ARTnumbers) : -1;
+                }
+                else
+                  return ARTcache - ARTnumbers;
+            }
+            bottom = ARTcache;
+        }
+        else if ( (ARTcache->ArtNum > i) && ((--ARTcache)->ArtNum < i) ) {
+            /* Missing article. */
+            return -1;
+        }
+        else {
+            ARTcache=NULL;
+            bottom = ARTnumbers;
+        }
     }
+    else
+        bottom=ARTnumbers;
 
+    /* The first hit when looping through is always bottom.
+     * This is a common case, and also happens to be the worst-case
+     * for this binary search.
+     * So take the gamble, and check it.
+     */
+    if (bottom->ArtNum >= i)
+    {
+        ARTcache=bottom;
+        if (bottom->ArtNum == i)
+        {
+            if (needcheck) {
+                return ARTinstore(bottom - ARTnumbers)
+                    ? (bottom-ARTnumbers) : -1;
+            }
+            return bottom-ARTnumbers;
+        }
+        else
+        {
+            /* The first article requested has already been cancelled. */
+            return -1;
+        }
+    }
     for ( ; ; ) {
 	if ((i < bottom->ArtNum) || (i > top->ArtNum))
 	    break;
@@ -342,8 +388,9 @@ STATIC BOOL ARTopen(char *name)
 	return FALSE;
 
     if ((ARTnumbers[i].ArtNum == artnum) && innconf->storageapi) {
-	if (ARTnumbers[i].Token.cancelled)
-	    return FALSE;
+        if (IsCancelledByIndex(i,artnum))
+            return FALSE;
+
 	if (ARTnumbers[i].Tokenretrieved)
 	    token = ARTnumbers[i].Token;
 	else {
@@ -353,12 +400,14 @@ STATIC BOOL ARTopen(char *name)
 		return FALSE;
 	    }
 	    UnpackOverIndex(*(ARTnumbers[i].Index), &index);
-	    ARTnumbers[i].Tokenretrieved = TRUE;
+	    if (HashEmpty(index.hash)) 
+		return FALSE;
 	    if ((tokentext = HISgetent(&index.hash, FALSE, NULL)) == (char *)NULL) {
 		ARTnumbers[i].Token.cancelled = TRUE;
 		return FALSE;
 	    }
 	    token = ARTnumbers[i].Token = TextToToken(tokentext);
+	    ARTnumbers[i].Tokenretrieved = TRUE;
 	}
 	if (token.type != TOKEN_EMPTY && token.index != OVER_NONE && !token.cancelled) {
 	    if ((ARThandle = SMretrieve(token, RETR_ALL)) == NULL) {
@@ -432,8 +481,9 @@ STATIC BOOL ARTopenbyid(char *msg_id, ARTNUM *ap)
 	*ap = 0;
     } else {
 	if (innconf->articlemmap) {
-	    if ((fd = open(p, O_RDONLY)) < 0)
+	    if ((fd = open(p, O_RDONLY)) < 0) {
 		return FALSE;
+	    }
 	    if ((fstat(fd, &Sb) < 0) || !S_ISREG(Sb.st_mode)) {
 		close(fd);
 		return FALSE;
@@ -445,15 +495,15 @@ STATIC BOOL ARTopenbyid(char *msg_id, ARTNUM *ap)
 	    }
 	    close(fd);
 	} else {
-	    if ((ARTqp = QIOopen(p)) == NULL)
+	    if ((ARTqp = QIOopen(p)) == NULL) {
 		return FALSE;
+	    }
 	    if (fstat(QIOfileno(ARTqp), &Sb) < 0 || !S_ISREG(Sb.st_mode)) {
 		ARTclose();
 		return FALSE;
 	    }
 	    CloseOnExec(QIOfileno(ARTqp), TRUE);
 	}
-	p += SPOOLlen + 1;
 	if ((q = strrchr(p, '/')) != NULL)
 	    *q++ = '\0';
 	if (GRPlast[0] && EQ(p, GRPlast))
@@ -462,6 +512,76 @@ STATIC BOOL ARTopenbyid(char *msg_id, ARTNUM *ap)
     return TRUE;
 }
 
+/*
+ * Figure out if an article has been cancelled.
+ * As a side-effect (and a very important one), sets
+ * the Token for the article.
+ * IsCancelled uses the article number proper.
+ * IsCancelledByIndex uses an index into ARTnumbers.
+ */
+STATIC BOOL IsCancelled(int artnum, BOOL checkart)
+{
+    int i;
+    
+    if (!innconf->storageapi)
+        return FALSE;
+
+    if ((i = ARTfind(artnum, checkart)) < 0)
+        return TRUE;
+
+    
+    return IsCancelledByIndex(i,artnum);
+}
+
+STATIC BOOL IsCancelledByIndex(int i, int artnum)
+{
+    OVERINDEX index;
+    char *tokentext;
+
+    if (!innconf->storageapi)
+        return FALSE;
+
+    if (ARTnumbers[i].Token.cancelled)
+        return TRUE;
+    if (ARTnumbers[i].Tokenretrieved) {
+        if (ARTnumbers[i].Token.type == TOKEN_EMPTY)
+            return TRUE;
+        return FALSE;
+    }
+
+    if ((artnum && (ARTnumbers[i].ArtNum != artnum)) || !ARTnumbers[i].Index)
+        return ARTnumbers[i].Token.cancelled = TRUE;
+    
+    UnpackOverIndex(*(ARTnumbers[i].Index), &index);
+    if (HashEmpty(index.hash)) 
+	return TRUE;
+    if (artnum && (index.artnum != artnum))
+        return ARTnumbers[i].Token.cancelled = TRUE;
+    
+#ifndef DO_TAGGED_HASH
+    if (innconf->extendeddbz) {
+        if (!OVERgetent(&index.hash, &ARTnumbers[i].Token, &ARTnumbers[i].Offset))
+            return ARTnumbers[i].Token.cancelled = TRUE;
+	return FALSE;
+    } else {
+        if (HISgetent(&index.hash, FALSE, &ARTnumbers[i].Offset) == (char *)NULL)
+            return ARTnumbers[i].Token.cancelled = TRUE;
+    }
+#else
+    /* need to call HISgetent here so ARTnumbers[i].Offset will be set for
+       code below. */
+    if (HISgetent(&index.hash, FALSE, &ARTnumbers[i].Offset) == (char *)NULL)
+            return ARTnumbers[i].Token.cancelled = TRUE;
+#endif
+
+    ARTnumbers[i].Tokenretrieved = TRUE;
+    if ((tokentext = HISgetent(&index.hash, TRUE, &ARTnumbers[i].Offset)) == (char *)NULL)
+      return ARTnumbers[i].Token.cancelled = TRUE;
+    
+    ARTnumbers[i].Token = TextToToken(tokentext);
+
+    return FALSE;
+}
 /*
 **  Send a (part of) a file to stdout, doing newline and dot conversion.
 */
@@ -498,7 +618,7 @@ STATIC void ARTsendmmap(SENDTYPE what)
         if (*p == '\n') {
             if (FirstLine)
                 FirstLine = 0;
-            if ((lastchar == '\n') && !InBody)
+            if ((lastchar == '\n') && !InBody) {
                 if (what == SThead) {
                     if (*(p-1) == '\r')
                         p--;
@@ -516,6 +636,7 @@ STATIC void ARTsendmmap(SENDTYPE what)
                         }
                     }
                 }
+	    }
             if (((what == STarticle) || 
                     ((what == SThead) && !InBody) || 
                     ((what == STbody) && InBody)) && 
@@ -639,8 +760,8 @@ char *GetHeader(char *header, BOOL IsLines)
 	    }
 	    if ((lastchar == '\n') || (p == ARTmem)) {
 		if (!strncasecmp(p, header, strlen(header))) {
-		    for (; (p < limit) && !isspace(*p) ; p++);
-		    for (; (p < limit) && isspace(*p) ; p++);
+		    for (; (p < limit) && !isspace((int)*p) ; p++);
+		    for (; (p < limit) && isspace((int)*p) ; p++);
 		    for (q = p; q < limit; q++) 
 			if ((*q == '\r') || (*q == '\n'))
 			    break;
@@ -744,7 +865,7 @@ FUNCTYPE CMDfetch(int ac, char *av[])
 		ARTsendmmap(what->Type);
 	    else
 		ARTsendqio(what->Type);
-    	    if (tart != 0 && RARTenable && RARTcount < ART_MAX)
+    	    if (tart != 0 && innconf->readertrack && RARTcount < ART_MAX)
 	        RARTtable[RARTcount++]=tart;
 	}
 	ARTclose();
@@ -794,7 +915,7 @@ FUNCTYPE CMDfetch(int ac, char *av[])
 	    ARTsendmmap(what->Type);
 	else
 	    ARTsendqio(what->Type);
-    	    if (tart != 0 && RARTenable && RARTcount < ART_MAX)
+    	    if (tart != 0 && innconf->readertrack && RARTcount < ART_MAX)
 	        RARTtable[RARTcount++]=tart;
     }
     if (ac > 1)
@@ -874,10 +995,7 @@ FUNCTYPE CMDnextlast(int ac, char *av[])
 
 STATIC BOOL CMDgetrange(int ac, char *av[], ARTRANGE *rp, BOOL *DidReply)
 {
-    char		*p, *tokentext;
-    OVERINDEX		index;
-    int			i;
-    ARTNUM		artnum;
+    char		*p;
 
     *DidReply = FALSE;
     if (GRPcount == 0) {
@@ -894,66 +1012,13 @@ STATIC BOOL CMDgetrange(int ac, char *av[], ARTRANGE *rp, BOOL *DidReply)
 	    return FALSE;
 	}
 	rp->High = rp->Low = ARTnumbers[ARTindex].ArtNum;
-	if (!innconf->storageapi)
-	    return TRUE;
-	if ((i = ARTfind(rp->High, FALSE)) < 0)
-	    return FALSE;
-	if (ARTnumbers[i].Token.cancelled)
-	    return FALSE;
-	if (ARTnumbers[i].Tokenretrieved) {
-	    if (ARTnumbers[i].Token.type == TOKEN_EMPTY)
-		return FALSE;
-	    return TRUE;
-	}
-	if ((ARTnumbers[i].ArtNum != rp->High) || !ARTnumbers[i].Index) {
-	    ARTnumbers[i].Token.cancelled = TRUE;
-	    return FALSE;
-	}
-	UnpackOverIndex(*(ARTnumbers[i].Index), &index);
-	if (index.artnum != rp->High) {
-	    ARTnumbers[i].Token.cancelled = TRUE;
-	    return FALSE;
-	}
-	ARTnumbers[i].Tokenretrieved = TRUE;
-	if ((tokentext = HISgetent(&index.hash, FALSE, NULL)) == (char *)NULL) {
-	    ARTnumbers[i].Token.cancelled = TRUE;
-	    return FALSE;
-	}
-	ARTnumbers[i].Token = TextToToken(tokentext);
-	return TRUE;
+        return !IsCancelled(rp->High, innconf->nnrpdcheckart);
     }
 
     /* Got just a single number? */
     if ((p = strchr(av[1], '-')) == NULL) {
 	rp->Low = rp->High = atol(av[1]);
-	if (!innconf->storageapi) {
-	    return TRUE;
-	}
-	if ((i = ARTfind(rp->Low, FALSE)) < 0)
-	    return FALSE;
-	if (ARTnumbers[i].Token.cancelled)
-	    return FALSE;
-	if (ARTnumbers[i].Tokenretrieved) {
-	    if (ARTnumbers[i].Token.type == TOKEN_EMPTY)
-		return FALSE;
-	    return TRUE;
-	}
-	if ((ARTnumbers[i].ArtNum != rp->Low) || !ARTnumbers[i].Index) {
-	    ARTnumbers[i].Token.cancelled = TRUE;
-	    return FALSE;
-	}
-	UnpackOverIndex(*(ARTnumbers[i].Index), &index);
-	if (index.artnum != rp->Low) {
-	    ARTnumbers[i].Token.cancelled = TRUE;
-	    return FALSE;
-	}
-	ARTnumbers[i].Tokenretrieved = TRUE;
-	if ((tokentext = HISgetent(&index.hash, FALSE, NULL)) == (char *)NULL) {
-	    ARTnumbers[i].Token.cancelled = TRUE;
-	    return FALSE;
-	}
-	ARTnumbers[i].Token = TextToToken(tokentext);
-	return TRUE;
+        return !IsCancelled(rp->Low, innconf->nnrpdcheckart);
     }
 
     /* Parse range. */
@@ -971,68 +1036,8 @@ STATIC BOOL CMDgetrange(int ac, char *av[], ARTRANGE *rp, BOOL *DidReply)
     else
 	/* No articles; make sure loops don't run. */
 	rp->High = rp->Low ? rp->Low - 1 : 0;
-    if (!innconf->storageapi)
-	return TRUE;
-    for (artnum = rp->Low; artnum <= rp->High; artnum++) {
-	if ((i = ARTfind(artnum, FALSE)) < 0)
-	    continue;
-	if (ARTnumbers[i].Token.cancelled)
-	    continue;
-	if (ARTnumbers[i].Token.type != TOKEN_EMPTY)
-	    continue;
-	if ((ARTnumbers[i].ArtNum != artnum) || !ARTnumbers[i].Index) {
-	    ARTnumbers[i].Token.cancelled = TRUE;
-	    continue;
-	}
-	UnpackOverIndex(*(ARTnumbers[i].Index), &index);
-	if (index.artnum != artnum) {
-	    ARTnumbers[i].Token.cancelled = TRUE;
-	    continue;
-	}
-#ifdef	DO_TAGGED_HASH
-	ARTnumbers[i].Tokenretrieved = TRUE;
-	if (HISgetent(&index.hash, FALSE, &ARTnumbers[i].Offset) == (char *)NULL) {
-	    ARTnumbers[i].Token.cancelled = TRUE;
-	    continue;
-	}
-	if ((tokentext = HISgetent(&index.hash, TRUE, &ARTnumbers[i].Offset)) == (char *)NULL) {
-	    ARTnumbers[i].Token.cancelled = TRUE;
-	    continue;
-	}
-	ARTnumbers[i].Token = TextToToken(tokentext);
-#else
-	if (innconf->extendeddbz) {
-	    if (!OVERgetent(&index.hash, &ARTnumbers[i].Token)) {
-		ARTnumbers[i].Token.cancelled = TRUE;
-		continue;
-	    }
-	} else {
-	    if (HISgetent(&index.hash, FALSE, &ARTnumbers[i].Offset) == (char *)NULL) {
-		ARTnumbers[i].Token.cancelled = TRUE;
-		continue;
-	    }
-	}
-#endif
-    }
-#ifdef	DO_TAGGED_HASH
+
     return TRUE;
-#else
-    if (innconf->extendeddbz)
-	return TRUE;
-    for (artnum = rp->Low; artnum <= rp->High; artnum++) {
-	if ((i = ARTfind(artnum, FALSE)) < 0)
-	    continue;
-	if (ARTnumbers[i].Token.cancelled)
-	    continue;
-	ARTnumbers[i].Tokenretrieved = TRUE;
-	if ((tokentext = HISgetent(&index.hash, TRUE, &ARTnumbers[i].Offset)) == (char *)NULL) {
-	    ARTnumbers[i].Token.cancelled = TRUE;
-	    continue;
-	}
-	ARTnumbers[i].Token = TextToToken(tokentext);
-    }
-    return TRUE;
-#endif
 }
 
 
@@ -1154,7 +1159,6 @@ STATIC BOOL OVERopen(void)
 */
 void OVERclose(void)
 {
-    int	i;
     if (innconf->storageapi) {
 	(void)OVERshutdown();
     } else {
@@ -1180,11 +1184,8 @@ void OVERclose(void)
 STATIC char *OVERfind(ARTNUM artnum, int *linelen)
 {
     int		        i;
-    char   	        *q;
-    OVERINDEX           index;
     STATIC char         *OVERline = NULL;
-    char		*tokentext, *nextline;
-    TOKEN		token;
+    char		*nextline;
 
     if (innconf->storageapi) {
 	if ((i = ARTfind(artnum, FALSE)) < 0)
@@ -1481,26 +1482,27 @@ FUNCTYPE CMDxhdr(int ac, char *av[])
 	}
 
     Reply("%d %s fields follow\r\n", NNTP_HEAD_FOLLOWS_VAL, av[1]);
-    for (i = range.Low; i <= range.High; i++) {
+    for (i = range.Low; i <= range.High && range.High > 0; i++) {
 	if (ARTfind(i, innconf->storageapi && innconf->nnrpdcheckart) < 0)
 	    continue;
 
 	/* Get it from the overview? */
-	if (Overview && (p = OVERfind(i, &linelen)) != NULL) {
+	if (Overview && !IsCancelled(i, FALSE) && (p = OVERfind(i, &linelen)) != NULL) {
 	    p = OVERGetHeader(p, Overview);
-	    Printf("%ld %s\r\n", i, p && *p ? p : "(none)");
+	    Printf("%d %s\r\n", i, p && *p ? p : "(none)");
 	    continue;
 	}
 
-	(void)sprintf(buff, "%ld", i);
+	(void)sprintf(buff, "%d", i);
 	if (!ARTopen(buff))
 	    continue;
 	p = GetHeader(av[1], IsLines);
-	Printf("%ld %s\r\n", i, p ? p : "(none)");
+	Printf("%d %s\r\n", i, p ? p : "(none)");
 	ARTclose();
     }
     Printf(".\r\n");
-    OVERclose();
+    if (innconf->storageapi)
+	OVERclose();
 }
 
 
@@ -1510,13 +1512,14 @@ FUNCTYPE CMDxhdr(int ac, char *av[])
 FUNCTYPE CMDxover(int ac, char *av[])
 {
     char	        *p;
-    ARTNUM	        i;
+    int	                i, j;
     BOOL	        Opened;
     BOOL	        DidReply;
     ARTRANGE		range;
     char		buff[SPOOLNAMEBUFF];
     struct timeval	stv, etv;
     int			linelen;
+    BOOL                checkart;
 
     if (!PERMcanread) {
 	Printf("%s\r\n", NOACCESS);
@@ -1541,26 +1544,32 @@ FUNCTYPE CMDxover(int ac, char *av[])
 
     OVERcount++;
     Reply("%d data follows\r\n", NNTP_OVERVIEW_FOLLOWS_VAL);
-    for (Opened = OVERopen(), i = range.Low; i <= range.High; i++) {
-	if (ARTfind(i, innconf->storageapi && innconf->nnrpdcheckart) < 0) {
-	    if (innconf->storageapi)
-		OVERmiss++;
+    /* Find the first article in the group that actually exists */
+    checkart = (innconf->storageapi && innconf->nnrpdcheckart);
+    for (; ((i = ARTfind(range.Low, checkart)) < 0) && (range.Low <= range.High);
+	range.Low++);
+
+    for (Opened = OVERopen();
+	 (i >= 0) && (i < ARTsize) && (ARTnumbers[i].ArtNum <= range.High) && (range.High > 0);
+	 i++) {
+	if (innconf->storageapi && !ARTinstore(i)) {
+	    OVERmiss++;
 	    continue;
 	}
 
-	if (Opened && (p = OVERfind(i, &linelen)) != NULL) {
+	if (Opened && (p = OVERfind(ARTnumbers[i].ArtNum, &linelen)) != NULL) {
 	    OVERhit++;
 	    OVERsize+=linelen;
 	    if ((innconf->storageapi && innconf->overviewmmap) || OVERmem) {
 		if (innconf->storageapi && innconf->overviewmmap) {
-		    (void)sprintf(buff, "%ld\t", i);
+		    (void)sprintf(buff, "%ld\t", ARTnumbers[i].ArtNum);
 		    SendIOb(buff, strlen(buff));
 		}
 		SendIOb(p, linelen);
 		SendIOb("\r\n", 2);
 	    } else {
 	        if (innconf->storageapi) {
-		    Printf("%ld\t%s\r\n", i, p);
+		    Printf("%ld\t%s\r\n", ARTnumbers[i].ArtNum, p);
 		} else {
 		    Printf("%s\r\n", p);
 		}
@@ -1571,7 +1580,7 @@ FUNCTYPE CMDxover(int ac, char *av[])
 	    continue;
 
 	/* This happens with traditional spool */
-	(void)sprintf(buff, "%ld", i);
+	(void)sprintf(buff, "%ld", ARTnumbers[i].ArtNum);
 	if ((p = OVERgen(buff)) != NULL) {
 	    OVERmiss++;
 	    linelen = strlen(p);
@@ -1590,7 +1599,8 @@ FUNCTYPE CMDxover(int ac, char *av[])
     } else {
 	Printf(".\r\n");
     }
-    OVERclose();
+    if (innconf->storageapi)
+	OVERclose();
     gettimeofday(&etv, NULL);
     OVERtime+=(etv.tv_sec - stv.tv_sec) * 1000;
     OVERtime+=(etv.tv_usec - stv.tv_usec) / 1000;
@@ -1662,31 +1672,33 @@ FUNCTYPE CMDxpat(int ac, char *av[])
 	}
 
     Printf("%d %s matches follow.\r\n", NNTP_HEAD_FOLLOWS_VAL, header);
-    for (pattern = Glom(&av[3]), i = range.Low; i <= range.High; i++) {
+    for (pattern = Glom(&av[3]), i = range.Low; i <= range.High && range.High > 0; i++) {
 	if (ARTfind(i, innconf->storageapi && innconf->nnrpdcheckart) < 0)
 	    continue;
 
 	/* Get it from the Overview? */
 	if (Overview
+         && !IsCancelled(i, FALSE)
 	 && (p = OVERfind(i, &linelen)) != NULL
 	 && (p = OVERGetHeader(p, Overview)) != NULL) {
 	    if (wildmat(p, pattern))
-		Printf("%ld %s\r\n", i, p);
+		Printf("%d %s\r\n", i, p);
 	    continue;
 	}
 
-	(void)sprintf(buff, "%ld", i);
+	(void)sprintf(buff, "%d", i);
 	
 	if (!ARTopen(buff))
 	    continue;
 	if ((p = GetHeader(av[1], FALSE)) == NULL)
 	    p = "(none)";
 	if (wildmat(p, pattern))
-	    Printf("%ld %s\r\n", i, p);
+	    Printf("%d %s\r\n", i, p);
 	ARTclose();
     }
 
     Printf(".\r\n");
     DISPOSE(pattern);
-    OVERclose();
+    if (innconf->storageapi)
+	OVERclose();
 }
