@@ -1,4 +1,4 @@
-/*  $Revision: 1.18 $
+/*  $Revision: 1.20.2.3 $
 **
 **  Miscellaneous commands.
 */
@@ -20,6 +20,9 @@ typedef struct _LISTINFO {
 } LISTINFO;
 
 
+extern int LLOGenable;
+
+
 STATIC LISTINFO		INFOactive = {
     NULL, _PATH_ACTIVE, TRUE, "active newsgroups",
     "Newsgroups in form \"group high low flags\""
@@ -206,6 +209,7 @@ CMDauthinfo(ac, av)
     static char	User[30];
     static char	Password[30];
     char	accesslist[BIG_BUFFER];
+    int         code;
 
     if (caseEQ(av[1], "generic")) {
 	char *logrec = Glom(av);
@@ -254,27 +258,61 @@ CMDauthinfo(ac, av)
 	(void)strncpy(Password, av[2], sizeof Password - 1);
 	Password[sizeof Password - 1] = 0;
 
-	if (EQ(User, PERMuser) && EQ(Password, PERMpass)) {
-	    syslog(L_NOTICE, "%s user %s", ClientHost, User);
-	    Reply("%d Ok\r\n", NNTP_AUTH_OK_VAL);
-	    PERMneedauth = FALSE;
-	    PERMauthorized = TRUE;
-	    return;
-	}
-	if (PERMinfile((char *)NULL, (char *)NULL, User, Password,
-		accesslist, NNRPACCESS)) {
-	    PERMspecified = NGgetlist(&PERMlist, accesslist);
-	    syslog(L_NOTICE, "%s user %s", ClientHost, User);
-	    Reply("%d Ok\r\n", NNTP_AUTH_OK_VAL);
-	    PERMneedauth = FALSE;
-	    PERMauthorized = TRUE;
-	    return;
+#ifdef DO_PERL
+	if (innconf->nnrpperlauth) {
+	    code = perlAuthenticate(ClientHost, ClientIp, User, Password, accesslist);
+	    if (code == NNTP_AUTH_OK_VAL) {
+		PERMspecified = NGgetlist(&PERMlist, accesslist);
+		syslog(L_NOTICE, "%s user %s", ClientHost, User);
+		Reply("%d Ok\r\n", NNTP_AUTH_OK_VAL);
+		PERMneedauth = FALSE;
+		PERMauthorized = TRUE;
+		return;
+	    } else {
+		syslog(L_NOTICE, "%s bad_auth", ClientHost);
+		Reply("%d Authentication error\r\n", NNTP_ACCESS_VAL);
+		ExitWithStats(1);
+	    }
+	} else {
+#endif /* DO_PERL */
+	    if (EQ(User, PERMuser) && EQ(Password, PERMpass)) {
+		syslog(L_NOTICE, "%s user %s", ClientHost, User);
+		if (LLOGenable) {
+			fprintf(locallog, "%s user (%s):%s\n", ClientHost, Username, User);
+			fflush(locallog);
+		}
+		if (LLOGenable) {
+			fprintf(locallog, "%s user (%s):%s\n", ClientHost, Username, User);
+			fflush(locallog);
+		}
+		Reply("%d Ok\r\n", NNTP_AUTH_OK_VAL);
+		/* save these values in case you need them later */
+		strcpy(PERMuser, User);
+		strcpy(PERMpass, Password);
+		PERMneedauth = FALSE;
+		PERMauthorized = TRUE;
+		return;
+	    }
+	    if (PERMinfile((char *)NULL, (char *)NULL, User, Password,
+			   accesslist, NNRPACCESS)) {
+		PERMspecified = NGgetlist(&PERMlist, accesslist);
+		syslog(L_NOTICE, "%s user %s", ClientHost, User);
+		if (LLOGenable) {
+			fprintf(locallog, "%s user (%s):%s\n", ClientHost, Username, User);
+			fflush(locallog);
+		}
+		Reply("%d Ok\r\n", NNTP_AUTH_OK_VAL);
+		PERMneedauth = FALSE;
+		PERMauthorized = TRUE;
+		return;
+	    }
+#ifdef DO_PERL
 	}
+#endif /* DO_PERL */
 
 	syslog(L_NOTICE, "%s bad_auth", ClientHost);
 	Reply("%d Authentication error\r\n", NNTP_ACCESS_VAL);
 	ExitWithStats(1);
-
     }
 
 }
@@ -341,7 +379,7 @@ CMDlist(ac, av)
 		grplist[0] = av[2];
 		grplist[1] = NULL;
 		Reply("%d list:\r\n", NNTP_LIST_FOLLOWS_VAL);
-		if (PERMmatch(PERMdefault, PERMlist, grplist))
+		if (PERMmatch(PERMlist, grplist))
 			Printf("%s %ld %ld %c%s\r\n.\r\n",
 		      		GPNAME(gp), (long)GPHIGH(gp), (long)GPLOW(gp),
 		      		GPFLAG(gp), GPALIAS(gp) ? GPALIAS(gp) : "");
@@ -400,7 +438,7 @@ CMDlist(ac, av)
     }
 
     Reply("%d %s.\r\n", NNTP_LIST_FOLLOWS_VAL, lp->Format);
-    if (!PERMspecified && !PERMdefault) {
+    if (!PERMspecified) {
 	/* Optmize for unlikely case of no permissions and FALSE default. */
 	(void)QIOclose(qp);
 	Printf(".\r\n");
@@ -429,7 +467,7 @@ CMDlist(ac, av)
 		    continue;
 		*save = '\0';
 		grplist[0] = q;
-		if (!PERMmatch(PERMdefault, PERMlist, grplist))
+		if (!PERMmatch(PERMlist, grplist))
 		    continue;
 		*save = ':';
 	    }
@@ -453,7 +491,7 @@ CMDlist(ac, av)
 	      
 	if (PERMspecified) {
 	    grplist[0] = p;
-	    if (!PERMmatch(PERMdefault, PERMlist, grplist))
+	    if (!PERMmatch(PERMlist, grplist))
 		continue;
 	}
 	if (wildarg && !wildmat(p, wildarg))
@@ -561,10 +599,10 @@ CMDnewgroups(ac, av)
 	if (PERMspecified) {
 	    grplist[0] = p;
 	    grplist[1] = NULL;
-	    if (!PERMmatch(PERMdefault, PERMlist, grplist))
+	    if (!PERMmatch(PERMlist, grplist))
 		continue;
 	}
-	else if (!PERMdefault)
+	else 
 	    continue;
 
 	if (!All) {
@@ -758,8 +796,11 @@ CMDxpath(ac, av)
     char	*p;
     HASH	hash = HashMessageID(av[1]);
 
-    if ((p = HISgetent(&hash, TRUE, NULL)) == NULL)
+    if (innconf->storageapi) {
+	Reply("%d Syntax error or bad command\r\n", NNTP_BAD_COMMAND_VAL);
+    } else if ((p = HISgetent(&hash, TRUE, NULL)) == NULL) {
 	Reply("%d Don't have it\r\n", NNTP_DONTHAVEIT_VAL);
-    else
+    } else {
 	Reply("%d %s\r\n", NNTP_NOTHING_FOLLOWS_VAL, p);
+    }
 }
