@@ -1,4 +1,4 @@
-/*  $Revision: 1.26 $
+/*  $Revision: 1.32.2.2 $
 **
 **  Newsgroups and the active file.
 */
@@ -43,7 +43,6 @@ STATIC int		NRequestID;
 
 unsigned int	RARTtable[ART_MAX];
 int		RARTcount=0;
-int		RARTenable=FALSE;
 int 		LLOGenable=FALSE;
 
 void
@@ -467,7 +466,10 @@ GetActivedGroupList()
     struct timeval timeout;
 
     if (GRPactived < 0) {
-	if ((s = create_udp_socket(0)) < 0) {
+	/* On Linux, the obtained socket can have the same ID (IP+port)	*/
+	/* than the actived server. In that case, any future nnrpd will	*/
+	/* be unable to connect to actived.				*/
+	if ((s = create_udp_socket(0, innconf->activedport)) < 0) {
 	    syslog(L_ERROR, "%s actived socket couldnt be created %m",
 				ClientHost);
 	    return(FALSE);
@@ -569,7 +571,7 @@ STATIC int ARTcompare(CPOINTER p1, CPOINTER p2)
 **  Fill in ARTnumbers with the numbers of the articles in the current
 **  group.
 */
-STATIC void GRPscandir(char *dir)
+STATIC void GRPscandir(char *dir, GROUPENTRY *ge)
 {
     static int		ARTarraysize;
     DIRENTRY	        *ep;
@@ -582,6 +584,10 @@ STATIC void GRPscandir(char *dir)
     char                (*tmp)[][OVERINDEXPACKSIZE];
     int                 icount;
     OVERINDEX           index;
+    ARTNUM		low, high;
+
+    low  = GPLOW(ge);
+    high = GPHIGH(ge);
 
     ARTsize = 0;
     GRPcount++;
@@ -614,7 +620,7 @@ STATIC void GRPscandir(char *dir)
 	    if (ARTarraysize == 0) {
 		ARTnumbers = NEW(ARTLIST, OVERicount);
 	    } else {
-		ARTnumbers = RENEW(ARTnumbers, ARTLIST, OVERicount);
+		RENEW(ARTnumbers, ARTLIST, OVERicount);
 	    }
 	    ARTarraysize = OVERicount;
 	    for (i = 0; i < OVERicount; i++) {
@@ -623,9 +629,11 @@ STATIC void GRPscandir(char *dir)
 		ARTnumbers[ARTsize].Token.cancelled = FALSE;
 		ARTnumbers[ARTsize].Token.type = TOKEN_EMPTY;
 		ARTnumbers[ARTsize].Tokenretrieved = FALSE;
+		ARTnumbers[ARTsize].Offset = -1;
 		ARTnumbers[ARTsize++].Index = &(*OVERindex)[i];
 	    }
-	}
+	} else
+	    close(fd);
 
     } else if (!innconf->storageapi) {
 	DISPOSE(path);
@@ -660,13 +668,15 @@ STATIC void GRPscandir(char *dir)
 	    }
 	    if (*p || i == 0)
 		continue;
-	    
+	    if (i < low || i > high)
+		continue;
 	    if (ARTsize + 1 >= ARTarraysize) {
 		ARTarraysize += 1024;
 		RENEW(ARTnumbers, ARTLIST, ARTarraysize);
 	    }
 	    
 	    ARTnumbers[ARTsize].ArtNum = i;
+	    ARTnumbers[ARTsize].Offset = -1;
 	    ARTnumbers[ARTsize++].Index = NULL;
 	}
 	(void)closedir(dp);
@@ -693,6 +703,7 @@ FUNCTYPE CMDgroup(int ac, char *av[])
     char		*grplist[2];
     char		*group;
     char		buff[SPOOLNAMEBUFF];
+    GROUPENTRY		*ge;
 
     if (!PERMcanread) {
 	Reply("%s\r\n", NOACCESS);
@@ -713,7 +724,7 @@ FUNCTYPE CMDgroup(int ac, char *av[])
     }
     else
 	group = av[1];
-    if (GRPfind(group) == NULL) {
+    if ((ge = GRPfind(group)) == NULL) {
 	Reply("%s %s\r\n", NOSUCHGROUP, group);
 	return;
     }
@@ -722,12 +733,11 @@ FUNCTYPE CMDgroup(int ac, char *av[])
     if (PERMspecified) {
 	grplist[0] = group;
 	grplist[1] = NULL;
-	if (!PERMmatch(PERMdefault, PERMlist, grplist)) {
+	if (!PERMmatch(PERMlist, grplist)) {
 	    Reply("%s %s\r\n", NOSUCHGROUP, group);
 	    return;
 	}
-    }
-    else if (!PERMdefault) {
+    } else {
 	Reply("%s %s\r\n", NOSUCHGROUP, group);
 	return;
     }
@@ -748,7 +758,7 @@ FUNCTYPE CMDgroup(int ac, char *av[])
     /* If we haven't been in the group recently, rescan. */
     (void)time(&now);
     if (!EQ(buff, GRPlast) || now > last_time + NNRP_RESCAN_DELAY) {
-	GRPscandir(buff);
+	GRPscandir(buff, ge);
 	(void)strcpy(GRPlast, buff);
 	last_time = now;
     }
@@ -794,7 +804,7 @@ GRPreport()
 	syslog(L_NOTICE, "%s group %s %ld", ClientHost, buff, GRParticles);
 	GRParticles = 0;
 	repbuff[0]='\0';
-	if (RARTenable && (RARTcount > 0)) {
+	if (innconf->readertrack && (RARTcount > 0)) {
 		for (pp=0;pp<RARTcount; pp++) {
 			sprintf(tmpbuff, "%ld", RARTtable[pp]);
 			strcat(repbuff, tmpbuff);
@@ -840,7 +850,7 @@ CMDxgtitle(ac, av)
     else
 	p = av[1];
 
-    if (!PERMdefault) {
+    if (!PERMspecified) {
 	Printf("%d list follows\r\n", NNTP_XGTITLE_OK);
 	Printf(".\r\n");
 	return;
@@ -864,7 +874,7 @@ CMDxgtitle(ac, av)
 	    if (PERMspecified) {
 		grplist[0] = line;
 		grplist[1] = NULL;
-		if (!PERMmatch(PERMdefault, PERMlist, grplist))
+		if (!PERMmatch(PERMlist, grplist))
 		    continue;
 	    }
 	    *q = save;
