@@ -1,4 +1,4 @@
-/*  $Revision: 1.22 $
+/*  $Revision: 1.28.2.4 $
 **
 **  Miscellaneous support routines.
 */
@@ -20,7 +20,7 @@ STATIC ino_t		ino = 0;
 
 #define ASCtoNUM(c)		((c) - '0')
 #define CHARStoINT(c1, c2)	(ASCtoNUM((c1)) * 10 + ASCtoNUM((c2)))
-#define DaysInYear(y)		((y % 4 ? 365 : 366))
+#define DaysInYear(y) ((y) % 4 ? 365 : (y) % 100 ? 366 : (y) % 400 ? 365 : 366)
 
 
 /*
@@ -97,14 +97,11 @@ Glom(av)
 **  Match a list of newsgroup specifiers against a list of newsgroups.
 **  func is called to see if there is a match.
 */
-BOOL
-PERMmatch(match, Pats, list)
-    register BOOL	match;
-    char		**Pats;
-    char		**list;
+BOOL PERMmatch(char **Pats, char **list)
 {
-    register int	i;
-    register char	*p;
+    int	                i;
+    char	        *p;
+    int                 match = FALSE;
 
     if (Pats[0] == NULL)
 	return TRUE;
@@ -138,7 +135,7 @@ PERMartok(qp)
     char		*p;
 
     if (!PERMspecified)
-	return PERMdefault;
+	return FALSE;
 
     if ((p = GetHeader("Newsgroups", FALSE)) == NULL)
         return 1;
@@ -146,7 +143,7 @@ PERMartok(qp)
 	/* No newgroups or null entry. */
 	return 1;
 
-    return PERMmatch(PERMdefault, PERMlist, grplist);
+    return PERMmatch(PERMlist, grplist);
 }
 
 
@@ -169,23 +166,45 @@ NNTPtoGMT(av1, av2)
     int			hour;
     int			mins;
     int			secs;
+    int			datelen;
     register int	i;
     long		seconds;
-    char		buff[6 + 6 + 1];
+    char		buff[8 + 6 + 1];
 
-    if (strlen(av1) != 6 || strlen(av2) != 6)
+    /* Y2K: accept YYMMDD, YYYMMDD or YYYYMMDD.
+            First is strict NNTP spec,
+	    Second is broken clients that do "printf %02d tm_year",
+	    Third is people trying to do the right date thing
+	          despite the spec. */
+    datelen = strlen(av1);
+    if ((datelen < 6 || datelen > 8) || strlen(av2) != 6)
 	return -1;
     (void)sprintf(buff, "%s%s", av1, av2);
     for (p = buff; *p; p++)
-	if (!CTYPE(isdigit, *p))
+	if (!CTYPE(isdigit, (int)*p))
 	    return -1;
 
-    year  = CHARStoINT(buff[ 0], buff[ 1]);
-    month = CHARStoINT(buff[ 2], buff[ 3]);
-    day   = CHARStoINT(buff[ 4], buff[ 5]);
-    hour  = CHARStoINT(buff[ 6], buff[ 7]);
-    mins  = CHARStoINT(buff[ 8], buff[ 9]);
-    secs  = CHARStoINT(buff[10], buff[11]);
+    p = buff + datelen - 6;
+
+    year  = CHARStoINT(p[ 0], p[ 1]);
+    month = CHARStoINT(p[ 2], p[ 3]);
+    day   = CHARStoINT(p[ 4], p[ 5]);
+    hour  = CHARStoINT(p[ 6], p[ 7]);
+    mins  = CHARStoINT(p[ 8], p[ 9]);
+    secs  = CHARStoINT(p[10], p[11]);
+
+    if (datelen == 6) {		/* YYMMDD */
+	if (year > 70)
+	    year += 1900;
+	else
+	    year += 2000;
+    } else {
+        year += ASCtoNUM(*--p) * 100;
+	if (datelen == 7)	/* YYYMMDD */
+	    year += 1900;
+	else
+	    year += ASCtoNUM(*--p) * 1000; /* YYYYMMDD */
+    }
 
     if (month < 1 || month > 12
      || day < 1 || day > 31
@@ -199,10 +218,7 @@ NNTPtoGMT(av1, av2)
     else if (hour < 0 || hour > 23)
 	return -1;
 
-    if (year < 50)
-      year += 100 ;
-    
-    for (seconds = 0, year += 1900, i = 1970; i < year; i++)
+    for (seconds = 0, i = 1970; i < year; i++)
 	seconds += DaysInYear(i);
     if (DaysInYear(year) == 366 && month > 2)
 	seconds++;
@@ -251,7 +267,7 @@ void HIScheck(void)
 /*
 **  Get overview offset
 */
-BOOL OVERgetent(HASH *key, TOKEN *token)
+BOOL OVERgetent(HASH *key, TOKEN *token, OFFSET_T *offset)
 {
     struct timeval	stv, etv;
     idxrec		ionevalue;
@@ -283,10 +299,11 @@ BOOL OVERgetent(HASH *key, TOKEN *token)
 	    OVERdbz+=(etv.tv_usec - stv.tv_usec) / 1000;
 	}
 	OVERmaketoken(token, iextvalue.offset[OVEROFFSET], iextvalue.overindex, iextvalue.overlen);
-    } else {
-	return FALSE;
+	if (offset)
+	    *offset = iextvalue.offset[HISTOFFSET];
+	return TRUE;
     }
-    return TRUE;
+    return FALSE;
 }
 #endif
 
@@ -294,7 +311,7 @@ BOOL OVERgetent(HASH *key, TOKEN *token)
 **  Return the path name of an article if it is in the history file.
 **  Return a pointer to static data.
 */
-char *HISgetent(HASH *key, BOOL flag, OFFSET_T *off)
+char *HISgetent(HASH *key, BOOL useoffset, OFFSET_T *off)
 {
     static char		path[BIG_BUFFER];
     char	        *p;
@@ -305,20 +322,25 @@ char *HISgetent(HASH *key, BOOL flag, OFFSET_T *off)
     struct stat		Sb;
     struct timeval	stv, etv;
     static int		entrysize = 0;
+    int			i;
 #ifndef DO_TAGGED_HASH
     idxrec		ionevalue;
     idxrecext		iextvalue;
 #endif
 
     if (entrysize == 0) {
-	HASH hash;
-	time_t dummy = ~(time_t)0;
-	TOKEN token;
-	sprintf(buff, "[%s]%c%lu%c%lu%c%lu%c%s\n", HashToText(hash),
-	    HIS_FIELDSEP, dummy, HIS_SUBFIELDSEP,
-	    dummy, HIS_SUBFIELDSEP,
-	    dummy, HIS_FIELDSEP, TokenToText(token));
-	entrysize = strlen(buff);
+	if (innconf->storageapi) {
+	    HASH hash;
+	    time_t dummy = ~(time_t)0;
+	    TOKEN token;
+	    sprintf(buff, "[%s]%c%lu%c%lu%c%lu%c%s\n", HashToText(hash),
+		HIS_FIELDSEP, dummy, HIS_SUBFIELDSEP,
+		dummy, HIS_SUBFIELDSEP,
+		dummy, HIS_FIELDSEP, TokenToText(token));
+	    entrysize = strlen(buff);
+	} else {
+	    entrysize = sizeof(buff) - 1;
+	}
     }
     if (!setup) {
 	if (!dbzinit(HISTORY)) {
@@ -329,7 +351,7 @@ char *HISgetent(HASH *key, BOOL flag, OFFSET_T *off)
     }
 
     /* Set the key value, fetch the entry. */
-    if (flag && (off != NULL)) {
+    if (useoffset && (off != NULL)) {
 	offset = *off;
 	gettimeofday(&stv, NULL);
     } else {
@@ -404,21 +426,14 @@ char *HISgetent(HASH *key, BOOL flag, OFFSET_T *off)
 	OVERseek+=(etv.tv_usec - stv.tv_usec) / 1000;
     }
     stv = etv;
-    if (flag && (off == NULL)) {
-	if (fgets(buff, sizeof buff, hfp) == NULL) {
-	    syslog(L_ERROR, "%s cant fgets from %ld %m", ClientHost, offset);
-	    return NULL;
-	}
-    } else {
-	if (read(fileno(hfp), buff, entrysize) < 0) {
-	    syslog(L_ERROR, "%s cant read from %ld %m", ClientHost, offset);
-	    return NULL;
-	}
-	buff[entrysize+1] = '\0';
-	if (strchr(buff, '\n') == NULL) {
-	    syslog(L_ERROR, "%s cant find end of line %ld %m", ClientHost, offset);
-	    return NULL;
-	}
+    if (read(fileno(hfp), buff, entrysize) < 0) {
+	syslog(L_ERROR, "%s cant read from %ld %m", ClientHost, offset);
+	return NULL;
+    }
+    buff[entrysize] = '\0';
+    if (strchr(buff, '\n') == NULL) {
+	syslog(L_ERROR, "%s cant find end of line %ld %m", ClientHost, offset);
+	return NULL;
     }
     if (innconf->nnrpdoverstats) {
 	gettimeofday(&etv, NULL);
@@ -438,17 +453,21 @@ char *HISgetent(HASH *key, BOOL flag, OFFSET_T *off)
 	return NULL;
     save = p + 1;
 
-    if (IsToken(save)) {
+    if (IsToken(save) && ((useoffset != TRUE) || (off != NULL))) {
 	strcpy(path, save);
 	return path;
     }
 
     /* Want the full data? */
-    if (flag) {
+    if ((useoffset == TRUE) && (off == NULL)) {
+	/* this is the case for called by CMDxpath() */
+	if (innconf->storageapi)
+	    return NULL;
 	(void)strcpy(path, save);
-	for (p = path; *p; p++)
+	for (p = path; *p; p++) {
 	    if (*p == '.')
 		*p = '/';
+	}
 	return path;
     }
 
@@ -636,20 +655,17 @@ InitBackoffConstants()
   
   /* Read the runtime config file to get parameters */
 
-  if (innconf->backoff_db == NULL) return;
-  else
-    (void)strncpy(postrec_dir,innconf->backoff_db,SMBUF);
+  if ((innconf->backoff_db == NULL) ||
+    !(innconf->backoff_k > 1L && innconf->backoff_postfast > 0L && innconf->backoff_postslow > 1L))
+    return;
 
   /* Need this database for backing off */
+  (void)strncpy(postrec_dir,innconf->backoff_db,SMBUF);
   if (stat(postrec_dir, &st) < 0) {
     syslog(L_ERROR, "%s cannot stat backoff_db '%s': %s",ClientHost,postrec_dir,strerror(errno));
     return;
   }
-
-  /* Only enable if the constants make sense */
-  if (innconf->backoff_k > 1L && innconf->backoff_postfast > 0L && innconf->backoff_postslow > 1L) {
-    BACKOFFenabled = TRUE;
-  }
+  BACKOFFenabled = TRUE;
 
   return;
 }
