@@ -1,4 +1,4 @@
-/*  $Revision: 1.43 $
+/*  $Revision: 1.49.2.3 $
 **
 **  NNTP server for readers (NNRP) for InterNetNews.
 **  This server doesn't do any real load-limiting, except for what has
@@ -18,10 +18,13 @@
 #include "nnrpd.h"
 #include <netdb.h>
 #include <pwd.h>
+#include <grp.h>
 #if	defined(HPUX)
 #include <sys/pstat.h>
 #endif	/* defined(HPUX) */
-
+#if HAVE_GETSPNAM
+#  include <shadow.h>
+#endif /* HAVE_GETSPNAM */
 
 /*
 ** Here is some defensive code to protect the news server from hosts,
@@ -35,6 +38,8 @@
 #define INITIAL_TIMEOUT	10
 #endif
 
+#define MAXPATTERNDEFINE	10
+
 #define CMDany		-1
 
 
@@ -54,8 +59,7 @@ char	*ACTIVETIMES = NULL;
 char	*HISTORY = NULL;
 char	*NEWSGROUPS = NULL;
 char	*NNRPACCESS = NULL;
-    /* Default permission -- change with adb. */
-BOOL	PERMdefault = FALSE;
+
 BOOL	ForceReadOnly = FALSE;
 char 	LocalLogFileName[256];
 
@@ -68,7 +72,9 @@ STATIC char	*TITLEend;
 #endif	/* !defined(HPUX) */
 STATIC SIGVAR	ChangeTrace;
 BOOL	DaemonMode = FALSE;
-
+#if HAVE_GETSPNAM
+STATIC char	*ShadowGroup;
+#endif
 
 extern FUNCTYPE	CMDauthinfo();
 extern FUNCTYPE	CMDdate();
@@ -88,7 +94,6 @@ extern FUNCTYPE	CMDxpat();
 extern FUNCTYPE	CMDxpath();
 extern FUNCTYPE	CMD_unimp();
 
-extern int RARTenable;
 extern int LLOGenable;
 extern int TrackClient();
 
@@ -175,7 +180,7 @@ ExitWithStats(x)
 	ClientHost, usertime, systime, STATfinish - STATstart);
     /* Tracking code - Make entries in the logfile(s) to show that we have
 	finished with this session */
-    if (RARTenable) {
+    if (innconf->readertrack) {
 	syslog(L_NOTICE, "%s Tracking Disabled (%s)", ClientHost, Username);
 	if (LLOGenable) {
 		fprintf(locallog, "%s Tracking Disabled (%s)\n", ClientHost, Username);
@@ -362,10 +367,15 @@ PERMinfile(hp, ip, user, pass, accesslist, accessfile)
     register char	*p;
     register BOOL	found;
     register int	i;
+    register int	lines;
     struct passwd	*pwd;
     char		buff[BIG_BUFFER];
+    char		*definelist[MAXPATTERNDEFINE];
     char		filename[SMBUF];
     char		*fields[5];
+#if HAVE_GETSPNAM
+    struct spwd		*spwd;
+#endif
 
     if ((F = fopen(accessfile, "r")) == NULL) {
 	syslog(L_ERROR, "%s cant fopen %s %m", ClientHost, accessfile);
@@ -378,7 +388,10 @@ PERMinfile(hp, ip, user, pass, accesslist, accessfile)
     found = FALSE;
     accesslist[0] = '\0';
     filename[0] = '\0';
+    lines = 0;
+    for (i=0;i<MAXPATTERNDEFINE;i++) definelist[i] = NULL;
     while (fgets(buff, sizeof buff, F) != NULL) {
+	lines++;
 	if ((p = strchr(buff, '\n')) != NULL)
 	    *p = '\0';
 	if ((p = strchr(buff, COMMENT_CHAR)) != NULL)
@@ -392,9 +405,21 @@ PERMinfile(hp, ip, user, pass, accesslist, accessfile)
 		*p = '\0';
 		fields[++i] = p + 1;
 	    }
-	if ((i != 4) && !((i == 1) && (*fields[1] == '/'))) {
+	if ((i != 4) && !((i == 1) && ((*fields[1] == '/') ||
+			(strncmp(fields[0],"%DEFINE", 7) == 0)))) {
 	    /* Malformed line. */
-	    syslog(L_ERROR, "Malformed line in access file");
+	    syslog(L_ERROR, "Malformed line %d in access file", lines);
+	    continue;
+	}
+
+	if ((*fields[0] == '%') && (strncmp(fields[0],"%DEFINE", 7) == 0)) {
+	    p = fields[0]; p += 7;
+	    i = atoi(p);
+	    if ((i < 0) || (i >= MAXPATTERNDEFINE)) {
+		syslog(L_ERROR, "Too many defines in access file (line %d)", lines);
+		continue;
+	    }
+	    definelist[i] = COPY(fields[1]);
 	    continue;
 	}
 
@@ -447,18 +472,21 @@ PERMinfile(hp, ip, user, pass, accesslist, accessfile)
 	} else
 	    filename[0] = '\0';
 
-#ifdef DONT_HAVE_SHADOW
 	/* See if we should lookup a specific user in the passwd file */
 	if (user && pass && EQ(fields[2], "+")) {
 	    if ((pwd = getpwnam(user)) == NULL)
 		continue;
+#if HAVE_GETSPNAM
+	    if ((spwd = getspnam(user)) != NULL)
+		pwd->pw_passwd = spwd->sp_pwdp;
+#endif
 	    if (!EQ(pwd->pw_passwd, crypt(pass, pwd->pw_passwd)))
 		continue;
-	} else
-#endif
-	/* Matching for a specific user or just the host? */
-	if (user && (!EQ(user, fields[2]) || !EQ(pass, fields[3])))
-	    continue;
+	} else {
+	    /* Matching for a specific user or just the host? */
+	    if (user && (!EQ(user, fields[2]) || !EQ(pass, fields[3])))
+		continue;
+	}
 
 	PERMcanread = strchr(fields[1], 'R') != NULL;
 	PERMcanpost = strchr(fields[1], 'P') != NULL;
@@ -467,10 +495,21 @@ PERMinfile(hp, ip, user, pass, accesslist, accessfile)
 	if (ForceReadOnly) PERMcanpost=FALSE;
 	(void)strcpy(PERMuser, user ? user : fields[2]);
 	(void)strcpy(PERMpass, pass ? pass : fields[3]);
-	(void)strcpy(accesslist, fields[4]);
+	if (*fields[4] == '%') {
+	    p = fields[4] + 1;
+	    i = atoi(p);
+	    if ((i < 0) || (i >= MAXPATTERNDEFINE) || (definelist[i] == NULL)) {
+		syslog(L_ERROR, "No definition %d in access file (line %d)", i, lines);
+		continue;
+	    }
+	    (void)strcpy(accesslist, definelist[i]);
+	} else 
+	    (void)strcpy(accesslist, fields[4]);
 	found = TRUE;
     }
     (void)fclose(F);
+    for (i=0;i<MAXPATTERNDEFINE;i++)
+       if (definelist[i] != NULL) DISPOSE(definelist[i]);
     if (found && (strlen(filename) > 0))
 	return(PERMinfile(hp, ip, user, pass, accesslist, filename));
     return found;
@@ -480,25 +519,26 @@ PERMinfile(hp, ip, user, pass, accesslist, accessfile)
 /*
 **  Determine access rights of the client.
 */
-STATIC void
-StartConnection(accesslist)
-    char		*accesslist;
+STATIC void StartConnection(char *accesslist)
 {
     struct sockaddr_in	sin;
     ARGTYPE		length;
     char		buff[SMBUF];
     char		*ClientAddr;
+    int                 code;
 
     /* Get the peer's name. */
     length = sizeof sin;
     ClientAddr = NULL;
     if (getpeername(STDIN, (struct sockaddr *)&sin, &length) < 0) {
-	if (!isatty(STDIN)) {
+#ifndef DEBUG
+      if (!isatty(STDIN)) {
 	    syslog(L_TRACE, "%s cant getpeername %m", "?");
             (void)strcpy(ClientHost, "?"); /* so stats generation looks correct. */
 	    Printf("%d I can't get your name.  Goodbye.\r\n", NNTP_ACCESS_VAL);
 	    ExitWithStats(1);
 	}
+#endif
 	(void)strcpy(ClientHost, "stdin");
         ClientIP = 0L;
     }
@@ -549,18 +589,33 @@ StartConnection(accesslist)
     LogName[sizeof(LogName) - 1] = '\0';
 
     syslog(L_NOTICE, "%s connect", ClientHost);
-    if (!PERMinfile(ClientHost, ClientAddr, (char *)NULL, (char *)NULL,
-	    accesslist, NNRPACCESS)) {
-	syslog(L_NOTICE, "%s no_access", ClientHost);
-	Printf("%d You are not in my access file.  Goodbye.\r\n",
-	    NNTP_ACCESS_VAL);
-	ExitWithStats(1);
+#ifdef DO_PERL
+    if (innconf->nnrpperlauth) {
+	if ((code = perlConnect(ClientHost, ClientIp, accesslist)) == 502) {
+	    syslog(L_NOTICE, "%s no_access", ClientHost);
+	    Printf("%d You are not in my access file. Goodbye.\r\n",
+		   NNTP_ACCESS_VAL);
+	    ExitWithStats(1);
+	}
+    } else {
+#endif	/* DO_PERL */
+	if (!PERMinfile(ClientHost, ClientAddr, (char *)NULL, (char *)NULL,
+			accesslist, NNRPACCESS)) {
+	    syslog(L_NOTICE, "%s no_access", ClientHost);
+	    Printf("%d You are not in my access file.  Goodbye.\r\n",
+		   NNTP_ACCESS_VAL);
+	    ExitWithStats(1);
+	}
+	PERMneedauth = PERMuser[0] != '\0' && PERMpass[0] != '\0';
+#ifdef DO_PERL
     }
+#endif /* DO_PERL */
 }
 
 
 #if	!defined(VAR_NONE)
 
+#if	!defined(VAR_NONE)
 #if	defined(VAR_VARARGS)
 #if	defined(lint)
 #define START_VARARG(fmt, vp, type)	va_start(vp); fmt = NULL
@@ -571,6 +626,7 @@ StartConnection(accesslist)
 #if	defined(VAR_STDARGS)
 #define START_VARARG(fmt, vp, type)	va_start(vp, fmt)
 #endif	/* defined(VAR_STDARGS) */
+#endif /* defined(VAR_NONE) */
 
 /*
 **  Send a reply, possibly with debugging output.
@@ -606,7 +662,7 @@ Reply(char *fmt, ...)
 	p = buff + strlen(buff) - 1;
 	while (p >= buff && (*p == '\n' || *p == '\r'))
 	    *p-- = '\0';
-	syslog(LOG_DEBUG, "%s > %s", ClientHost, buff);
+	syslog(L_TRACE, "%s > %s", ClientHost, buff);
 
 	va_end(vp);
 	errno = oerrno;
@@ -654,6 +710,28 @@ WaitChild(s)
     (void)signal(s, WaitChild);
 }
 
+STATIC void SetupDaemon(void) {
+    BOOL                val;
+    
+#if defined(DO_PERL)
+    /* Load the Perl code */
+    PERLsetup(NULL, cpcatpath(innconf->pathfilter, _PATH_PERL_FILTER_NNRPD), "filter_post");
+    if (innconf->nnrpperlauth) {
+	PERLsetup(NULL, cpcatpath(innconf->pathfilter, _PATH_PERL_AUTH), "authenticate");
+	PerlFilter(TRUE);
+	perlAuthInit();
+    } else {
+	PerlFilter(TRUE);
+    }
+#endif /* defined(DO_PERL) */
+    
+    val = TRUE;
+    if (innconf->storageapi && SMsetup(SM_PREOPEN, (void *)&val) && !SMinit()) {
+	syslog(L_NOTICE, "%s cant initialize storage method, %s", ClientHost, SMerrorstr);
+	Reply("%d NNTP server unavailable. Try later.\r\n", NNTP_TEMPERR_VAL);
+	ExitWithStats(1);
+    }
+}
 
 /*
 **  Print a usage message and exit.
@@ -702,6 +780,10 @@ main(argc, argv, env)
     UID_T               NewsUID;
     int                 one = 1;
     FILE                *pidfile;
+#if HAVE_GETSPNAM
+    struct group	*grp;
+    GID_T		shadowgid;
+#endif /* HAVE_GETSPNAM */
 
 #if	!defined(HPUX)
     /* Save start and extent of argv for TITLEset. */
@@ -712,14 +794,13 @@ main(argc, argv, env)
     /* Parse arguments.   Must COPY() optarg if used because the
      * TITLEset() routine would clobber it! */
     Reject = NULL;
-    RARTenable=FALSE;
     LLOGenable=FALSE;
 
     openlog("nnrpd", L_OPENLOG_FLAGS | LOG_PID, LOG_INN_PROG);
 
     if (ReadInnConf() < 0) exit(1);
 
-    while ((i = getopt(argc, argv, "b:Di:lop:Rr:s:t")) != EOF)
+    while ((i = getopt(argc, argv, "b:Di:g:op:Rr:s:t")) != EOF)
 	switch (i) {
 	default:
 	    Usage();
@@ -733,12 +814,14 @@ main(argc, argv, env)
  	case 'D':			/* standalone daemon mode */
  	    DaemonMode = TRUE;
  	    break;
+#if HAVE_GETSPNAM
+	case 'g':
+	    ShadowGroup = optarg;
+	    break;
+#endif /* HAVE_GETSPNAM */
 	case 'i':			/* Initial command */
 	    PushedBack = COPY(optarg);
 	    break;
-	case 'l':			/* Tracking */
-	    RARTenable=TRUE;
-	    break;
 	case 'o':
 	    Offlinepost = TRUE;		/* Offline posting only */
 	    break;
@@ -795,6 +878,32 @@ main(argc, argv, env)
 		syslog(L_FATAL, "nnrpd cant stat %s %m", innconf->pathrun);
 		exit(1);
 	    }
+
+#if HAVE_GETSPNAM
+	    shadowgid = -1;
+	    /* Find shadowgroup gid if needed */
+	    if (ShadowGroup != NULL) {
+		if ((grp = getgrnam(ShadowGroup)) == NULL)
+		    syslog(L_ERROR, "nnrpd cannot find group %s",
+				ShadowGroup);
+		else
+		    shadowgid = grp->gr_gid;
+	    } else if ((grp = getgrnam("shadow")) != NULL) {
+		/* found default group "shadow" */
+		shadowgid = grp->gr_gid;
+		ShadowGroup = "shadow";
+	    }
+	    /* If we have a shadowgid, try to set it as an extra group. */
+	    if (shadowgid >= 0) {
+		if (setgroups(1, &shadowgid) < 0)
+		   syslog(L_ERROR, "nnrpd cannot set supplementary group %s %m",
+			ShadowGroup);
+		else
+		   syslog(L_NOTICE, "nnrpd added supplementary group %s",
+			ShadowGroup);
+	    }
+#endif /* HAVE_GETSPNAM */
+
 	    NewsUID = Sb.st_uid;
 	    NewsGID = Sb.st_gid;
 	    (void)setgid(NewsGID);
@@ -806,8 +915,8 @@ main(argc, argv, env)
 	}
 
 	/* Detach */
-	if ((pid = FORK()) < 0) {
-	    fprintf(stderr, "%s: can't ford (%s)\n", argv[0], strerror(errno));
+	if ((pid = fork()) < 0) {
+	    fprintf(stderr, "%s: can't fork (%s)\n", argv[0], strerror(errno));
 	    syslog(L_FATAL, "can't fork (%m)");
 	    exit(1);
 	} else if (pid != 0) 
@@ -825,6 +934,7 @@ main(argc, argv, env)
 
 	/* Set signal handle to care for dead children */
 	(void)signal(SIGCHLD, WaitChild);
+	SetupDaemon();
  
 	TITLEset("nnrpd: accepting connections");
  	
@@ -836,7 +946,7 @@ listen_loop:
 	if (fd < 0)
 		goto listen_loop;
     
-	for (i = 0; (pid = FORK()) < 0; i++) {
+	for (i = 0; (pid = fork()) < 0; i++) {
 	    if (i == MAX_FORKS) {
 		syslog(L_FATAL, "cant fork %m -- giving up");
 		exit(1);
@@ -857,8 +967,13 @@ listen_loop:
 	close(fd);
 	dup2(0, 1);
 	dup2(0, 2);
+
+	/* Only automatically reap children in the listening process */
+	(void)signal(SIGCHLD, SIG_DFL);
  
-    }  /* DaemonMode */
+    } else {
+	SetupDaemon();
+    }/* DaemonMode */
 
     /* Setup. */
     if (GetTimeInfo(&Now) < 0) {
@@ -899,7 +1014,7 @@ listen_loop:
 
     /* Get permissions and see if we can talk to this client */
     StartConnection(accesslist);
-    if (!PERMcanread && !PERMcanpost) {
+    if (!PERMcanread && !PERMcanpost && !PERMneedauth) {
 	syslog(L_NOTICE, "%s no_permission", ClientHost);
 	Printf("%d You have no permission to talk.  Goodbye.\r\n",
 	       NNTP_ACCESS_VAL);
@@ -907,7 +1022,6 @@ listen_loop:
     }
 
     /* Proceed with initialization. */
-    PERMneedauth = PERMuser[0] != '\0' && PERMpass[0] != '\0';
     PERMspecified = NGgetlist(&PERMlist, accesslist);
     TITLEset("connect");
 
@@ -918,10 +1032,10 @@ listen_loop:
 	ExitWithStats(0);
     }
 
-    if (RARTenable)
-	RARTenable=TrackClient(ClientHost,Username);
+    if (innconf->readertrack)
+	innconf->readertrack=TrackClient(ClientHost,Username);
 
-    if (RARTenable) {
+    if (innconf->readertrack) {
 	syslog(L_NOTICE, "%s Tracking Enabled (%s)", ClientHost, Username);
 	pid=getpid();
 	gettimeofday(&tv,NULL);
@@ -946,18 +1060,6 @@ listen_loop:
 	ExitWithStats(1);
     }
 
-    if (innconf->storageapi && !SMinit()) {
-	syslog(L_NOTICE, "%s cant initialize storage method, %s", ClientHost, SMerrorstr);
-	Reply("%d NNTP server unavailable. Try later.\r\n", NNTP_TEMPERR_VAL);
-	ExitWithStats(1);
-    }
-
-#if defined(DO_PERL)
-    /* Load the Perl code */
-    PERLsetup(NULL, cpcatpath(innconf->pathfilter, _PATH_PERL_FILTER_NNRPD), "filter_post");
-    PerlFilter (TRUE) ;
-#endif /* defined(DO_PERL) */
-
     Reply("%d %s InterNetNews NNRP server %s ready (%s).\r\n",
 	   PERMcanpost ? NNTP_POSTOK_VAL : NNTP_NOPOSTOK_VAL,
 	   innconf->pathhost, INNVersion(),
