@@ -1,4 +1,4 @@
-/*  $Revision: 1.22 $
+/*  $Revision: 1.28.2.5 $
 **
 **  Check article, send it to the local server.
 */
@@ -18,7 +18,6 @@
 #define FLUSH_ERROR(F)		(fflush((F)) == EOF || ferror((F)))
 #define HEADER_DELTA		20
 
-extern int RARTenable;
 extern int LLOGenable;
 
 STATIC char     *tmpPtr ;
@@ -168,7 +167,7 @@ TrimSpaces(p)
 
     for (start = p; ISWHITE(*start); start++)
 	continue;
-    for (p = start + strlen(start); p > start && CTYPE(isspace, p[-1]); )
+    for (p = start + strlen(start); p > start && CTYPE(isspace, (int)p[-1]); )
 	*--p = '\0';
     return start;
 }
@@ -209,7 +208,7 @@ StripOffHeaders(article)
     for (p = article; ; ) {
 
 	/* See if it's a known header. */
-	c = CTYPE(islower, *p) ? toupper(*p) : *p;
+	c = CTYPE(islower, (int)*p) ? toupper(*p) : *p;
 	for (hp = Table; hp < ENDOF(Table); hp++)
 	    if (c == hp->Name[0]
 	     && p[hp->Size] == ':'
@@ -363,7 +362,7 @@ ProcessHeaders(linecount)
      * unauthenticated data. */
     if (PERMauthorized && HDR(_sender) == NULL) {
 	(void)sprintf(sendbuff, "%s@%s",
-	    PERMuser ? PERMuser : "UNKNOWN", ClientHost);
+	    PERMuser[0] ? PERMuser : "UNKNOWN", ClientHost);
 	HDR(_sender) = sendbuff;
     }
     else if (!PERMauthorized)
@@ -575,7 +574,7 @@ CheckIncludedText(p, lines)
 	    i--;
 	    break;
 	}
-	if ((p = strchr(p + 1, '\n')) == NULL)
+	if ((p = strchr(p, '\n')) == NULL)
 	    break;
     }
     if ((i * 2 > lines) && (lines > 40))
@@ -605,8 +604,10 @@ MailArticle(group, article)
     if ((address = GetModeratorAddress(NULL, NULL, group)) == NULL) {
 	(void)sprintf(Error, "No mailing address for \"%s\" -- %s",
 		group, "ask your news administrator to fix this");
+	DISPOSE(group);  
 	return Error;
     }
+    DISPOSE(group);
 
     /* Now build up the command (ignore format/argument mismatch errors,
      * in case %s isn't in inconf->mta) and send the headers. */
@@ -661,9 +662,9 @@ MailArticle(group, article)
 **  moderated, etc.
 */
 STATIC STRING
-ValidNewsgroups(hdr, article)
+ValidNewsgroups(hdr, modgroup)
     char		*hdr;
-    char		*article;
+    char		**modgroup;
 {
     static char		distbuff[SMBUF];
     register char	*groups;
@@ -682,16 +683,21 @@ ValidNewsgroups(hdr, article)
 	return "Can't parse newsgroups line";
 
     /* Don't mail article if just checking Followup-To line. */
-    approved = HDR(_approved) != NULL || article == NULL;
+    approved = HDR(_approved) != NULL || modgroup == NULL;
 
     Error[0] = '\0';
     FoundOne = FALSE;
     h = DDstart((FILE *)NULL, (FILE *)NULL);
     do {
-#if	defined(DO_MERGE_TO_GROUPS)
-	if (p[0] == 't' && p[1] == 'o' && p[2] == '.')
+	if (innconf->mergetogroups && p[0] == 't' && p[1] == 'o' && p[2] == '.')
 	    p = "to";
-#endif	/* defined(DO_MERGE_TO_GROUPS) */
+        if (PERMspecified) {
+	    grplist[0] = p;
+	    grplist[1] = NULL;
+	    if (!PERMmatch(PERMlist, grplist)) {
+		sprintf(Error, "You are not allowed to post to %s\r\n", p);
+	    }
+        }
 	if ((gp = GRPfind(p)) == NULL)
 	    continue;
 	FoundOne = TRUE;
@@ -700,11 +706,8 @@ ValidNewsgroups(hdr, article)
 	case NF_FLAG_OK:
 	    break;
 	case NF_FLAG_MODERATED:
-	    if (!approved) {
-		DISPOSE(groups);
-		tmpPtr = DDend(h);
-		DISPOSE(tmpPtr);
-		return MailArticle(GPNAME(gp), article);
+	    if (!approved && !*modgroup) {
+		*modgroup = COPY(GPNAME(gp));
 	    }
 	    break;
 	case NF_FLAG_IGNORE:
@@ -722,13 +725,6 @@ ValidNewsgroups(hdr, article)
 		    p, GPALIAS(gp));
 	    break;
 	}
-        if (PERMspecified) {
-	    grplist[0] = p;
-	    grplist[1] = NULL;
-	    if (!PERMmatch(PERMdefault, PERMlist, grplist)) {
-		sprintf(Error,"%s: %s\r\n", NNTP_NOSUCHGROUP,p);
-	    }
-        }
     } while ((p = strtok((char *)NULL, NGSEPS)) != NULL);
     DISPOSE(groups);
 
@@ -794,9 +790,10 @@ OfferArticle(buff, buffsize, FromServer, ToServer)
 **  Spool article to temp file.
 */
 STATIC STRING
-Spoolit(article, Error)
+SpoolitTo(article, Error, SpoolDir)
     char 		*article;
     char		*Error;
+    char                *SpoolDir;
 {
     static char		CANTSPOOL[NNTP_STRLEN+2];
     register HEADER	*hp;
@@ -809,7 +806,7 @@ Spoolit(article, Error)
     sprintf(CANTSPOOL, "%s and can't write text to local spool file", Error);
 
     /* Try to write it to the spool dir. */
-    TempName(innconf->pathincoming, temp);
+    TempName(SpoolDir, temp);
     /* rnews -U ignores files starting with . */
     strrchr(temp, '/')[1] = '.';
     if ((F = fopen(temp, "w")) == NULL) {
@@ -847,7 +844,7 @@ Spoolit(article, Error)
     if (fclose(F))
 	return CANTSPOOL;
 
-    TempName(innconf->pathincoming, path);
+    TempName(SpoolDir, path);
     if (rename(temp, path) < 0) {
         syslog(L_FATAL, "cant rename %s %s %m", temp, path);
 	return CANTSPOOL;
@@ -857,6 +854,18 @@ Spoolit(article, Error)
     return NULL;
 }
 
+/*
+**  Spool article to temp file.
+*/
+STATIC STRING
+Spoolit(article, Error)
+    char 		*article;
+    char		*Error;
+{
+    return SpoolitTo(article, Error, innconf->pathincoming);
+}
+ 
+
 STRING
 ARTpost(article, idbuff)
     char		*article;
@@ -870,10 +879,12 @@ ARTpost(article, idbuff)
     FILE		*ToServer;
     FILE		*FromServer;
     char		buff[NNTP_STRLEN + 2], frombuf[SMBUF];
+    char		*modgroup = NULL;
     STRING		error;
     char		TrackID[NNTP_STRLEN];
     FILE		*ftd;
     int			result;
+    char		SDir[255];
 
     sprintf(TrackID,"%s/trackposts/track.", innconf->pathlog);
 
@@ -904,13 +915,8 @@ ARTpost(article, idbuff)
     if (i == 0 && HDR(_control) == NULL)
 	return "Article is empty";
 
-    if (idbuff != NULL)
-      strcpy (idbuff,"(mailed to moderator)") ;
-    if ((error = ValidNewsgroups(HDR(_newsgroups), article)) != NULL
-     || WasMailed)
+    if ((error = ValidNewsgroups(HDR(_newsgroups), &modgroup)) != NULL)
 	return error;
-    if (idbuff != NULL)
-      idbuff [0] = '\0' ;
     
     strncpy(frombuf, HDR(_from), sizeof(frombuf) - 1);
     frombuf[sizeof(frombuf) - 1] = '\0';
@@ -924,29 +930,79 @@ ARTpost(article, idbuff)
     if (p) {
 	strcpy(frombuf, p+1);
 	p = strrchr(frombuf, '.');
-	if (!p)
+	if (!p) {
+	    if (modgroup)
+		DISPOSE(modgroup);
 	    return "From: address not in Internet syntax";
+	}
     }
-    else
+    else {
+	if (modgroup)
+	    DISPOSE(modgroup);
 	return "From: address not in Internet syntax";
+    }
     if ((p = HDR(_followupto)) != NULL
      && !EQ(p, "poster")
-     && (error = ValidNewsgroups(p, (char *)NULL)) != NULL)
+     && (error = ValidNewsgroups(p, (char **)NULL)) != NULL) {
+	if (modgroup)
+	    DISPOSE(modgroup);
 	return error;
+    }
     if ((innconf->localmaxartsize > 0) &&
 		(strlen(article) > innconf->localmaxartsize)) {
 	    (void)sprintf(Error,
 		"Article is bigger then local limit of %ld bytes\n",
 		atoi(p));
+	    if (modgroup)
+		DISPOSE(modgroup);
 	    return Error;
     }
 
 #if defined(DO_PERL)
     /* Calls the Perl subroutine for headers management */
-    if ((p = (char *)HandleHeaders(article)) != NULL)
-        return p;
+    if ((p = (char *)HandleHeaders(article)) != NULL) {
+	if (idbuff) {
+	    if (modgroup)
+		sprintf(idbuff, "(mailed to moderator for %s)", modgroup);
+	    else
+		(void)strncpy(idbuff, HDR(_messageid), SMBUF - 1);
+	    idbuff[SMBUF - 1] = '\0';
+	}
+	if (strncmp(p, "DROP", 4) == 0) {
+	    syslog(L_NOTICE, "%s post %s", ClientHost, p);
+	    if (modgroup)
+		DISPOSE(modgroup);
+	    return NULL;
+	}
+	else if (strncmp(p, "SPOOL", 5) == 0) {
+	    syslog(L_NOTICE, "%s post %s", ClientHost, p);
+	    strcpy(SDir, innconf->pathincoming);
+	    if (modgroup)
+	    {
+		DISPOSE(modgroup);
+		return SpoolitTo(article, p, strcat(SDir,"/spam/mod"));
+	    }
+	    else
+		return SpoolitTo(article, p, strcat(SDir,"/spam"));
+	}
+	else
+	{
+	    if (modgroup)
+		DISPOSE(modgroup);
+	    return p;
+	}
+    }
 #endif /* defined(DO_PERL) */
 
+    /* handle mailing to moderated groups */
+
+    if (modgroup)
+    {
+      if (idbuff != NULL)
+          strcpy (idbuff,"(mailed to moderator)") ;
+      return MailArticle(modgroup, article);
+    }
+
     if (innconf->spoolfirst)
 	return Spoolit(article, Error);
 
@@ -955,12 +1011,14 @@ ARTpost(article, idbuff)
 
     /* Open a local connection to the server. */
     if (innconf->nnrpdposthost != NULL)
-	i = NNTPconnect(innconf->nnrpdposthost, NNTP_PORT, &FromServer, &ToServer, buff);
+	i = NNTPconnect(innconf->nnrpdposthost, innconf->nnrpdpostport,
+					&FromServer, &ToServer, buff);
     else {
 #if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
 	i = NNTPlocalopen(&FromServer, &ToServer, buff);
 #else
-	i = NNTPremoteopen(NNTP_PORT, &FromServer, &ToServer, buff);
+	i = NNTPremoteopen(innconf->port, &FromServer,
+					&ToServer, buff);
 #endif	/* defined(HAVE_UNIX_DOMAIN_SOCKETS) */
     }
 
@@ -1038,7 +1096,7 @@ ARTpost(article, idbuff)
     }
 
     /* Tracking */
-    if (RARTenable) {
+    if (innconf->readertrack) {
 	strcat(TrackID,HDR(_messageid));
 	if ((ftd=fopen(TrackID,"w")) != NULL) {
 		for (hp = Table; hp < ENDOF(Table); hp++)
