@@ -1,4 +1,4 @@
-/*  $Id: cnfs.c,v 1.39 1998/07/23 00:00:30 kondou Exp $
+/*  $Id: cnfs.c,v 1.51.2.6 1999/01/09 17:38:43 kondou Exp $
 **
 **  cyclic news file system
 */
@@ -43,6 +43,7 @@ STATIC METACYCBUFF 	*metacycbufftab = (METACYCBUFF *)NULL;
 STATIC CNFSEXPIRERULES	*metaexprulestab = (CNFSEXPIRERULES *)NULL;
 STATIC long		pagesize = 0;
 STATIC int		metabuff_update = METACYCBUFF_UPDATE;
+STATIC int		refresh_interval = REFRESH_INTERVAL;
 
 STATIC TOKEN CNFSMakeToken(char *cycbuffname, CYCBUFF_OFF_T offset,
 		       U_INT32_T cycnum, STORAGECLASS class, TOKEN *oldtoken) {
@@ -85,7 +86,7 @@ STATIC BOOL CNFSBreakToken(TOKEN token, char *cycbuffname,
     memcpy(cycbuffname, token.token, CNFSMAXCYCBUFFNAME);
     *(cycbuffname + CNFSMAXCYCBUFFNAME) = '\0';	/* Just to be paranoid */
     memcpy(&int32, &token.token[8], sizeof(int32));
-    *offset = ntohl(int32) * CNFS_BLOCKSIZE;
+    *offset = (CYCBUFF_OFF_T)ntohl(int32) * (CYCBUFF_OFF_T)CNFS_BLOCKSIZE;
     memcpy(&int32, &token.token[12], sizeof(int32));
     *cycnum = ntohl(int32);
     return TRUE;
@@ -159,7 +160,7 @@ STATIC CYCBUFF_OFF_T CNFShex2offt(char *hex) {
 		break;
 	    }
 	    n += (*hex - diff);
-	    if (isalnum(*(hex + 1)))
+	    if (isalnum((int)*(hex + 1)))
 		n <<= 4;
 	}
 	return n;
@@ -351,6 +352,7 @@ STATIC BOOL CNFSparse_part_line(char *l) {
   cycbuff->fd = -1;
   cycbuff->next = (CYCBUFF *)NULL;
   cycbuff->needflush = FALSE;
+  cycbuff->bitfield = (caddr_t)NULL;
   /*
   ** The minimum article offset will be the size of the bitfield itself,
   ** len / (blocksize * 8), plus however many additional blocks the CYCBUFF
@@ -373,7 +375,6 @@ STATIC BOOL CNFSparse_part_line(char *l) {
 
 STATIC BOOL CNFSparse_metapart_line(char *l) {
   char		*p, *cycbuff;
-  int		rpi;
   CYCBUFF	*rp;
   METACYCBUFF	*metacycbuff, *tmp;
 
@@ -451,13 +452,13 @@ STATIC BOOL CNFSparse_groups_line() {
   sub = SMGetConfig(TOKEN_CNFS, sub);
   for (;sub != (STORAGE_SUB *)NULL; sub = SMGetConfig(TOKEN_CNFS, sub)) {
     if (sub->options == (char *)NULL) {
-      syslog(L_ERROR, "%s: storage.ctl additional field is missing",
+      syslog(L_ERROR, "%s: storage.conf options field is missing",
 	   LocalLogName);
       CNFScleanexpirerule();
       return FALSE;
     }
     if ((mrp = CNFSgetmetacycbuffbyname(sub->options)) == NULL) {
-      syslog(L_ERROR, "%s: storage.ctl additional field '%s' undefined",
+      syslog(L_ERROR, "%s: storage.conf options field '%s' undefined",
 	   LocalLogName, sub->options);
       CNFScleanexpirerule();
       return FALSE;
@@ -491,7 +492,7 @@ STATIC BOOL CNFSparse_groups_line() {
 STATIC BOOL CNFSinit_disks(CYCBUFF *cycbuff) {
   char		buf[64];
   CYCBUFFEXTERN	rpx;
-  int		i, fd, bytes;
+  int		fd, bytes;
   CYCBUFF_OFF_T	tmpo;
   BOOL		oneshot;
 
@@ -551,8 +552,7 @@ STATIC BOOL CNFSinit_disks(CYCBUFF *cycbuff) {
 	if (strncmp(rpx.path, cycbuff->path, CNFSPASIZ) != 0) {
 	    syslog(L_ERROR, "%s: Path mismatch: read %s for cycbuff %s",
 		   LocalLogName, rpx.path, cycbuff->path);
-	    return FALSE;
-	}
+	} 
 	strncpy(buf, rpx.lena, CNFSLASIZ);
 	buf[CNFSLASIZ] = '\0';
 	tmpo = CNFShex2offt(buf);
@@ -611,7 +611,8 @@ STATIC BOOL CNFSread_config(void) {
     int		ctab_i;
     BOOL	metacycbufffound = FALSE;
     BOOL	cycbuffupdatefound = FALSE;
-    int		update;
+    BOOL	refreshintervalfound = FALSE;
+    int		update, refresh;
 
     if ((config = ReadInFile(cpcatpath(innconf->pathetc, _PATH_CYCBUFFCONFIG),
 				(struct stat *)NULL)) == NULL) {
@@ -640,7 +641,7 @@ STATIC BOOL CNFSread_config(void) {
 	while (1) {
 	    if (*from && *from == '\\' && *(from + 1) == '\n') {
 		from += 2;		/* Skip past backslash+newline */
-		while (*from && isspace(*from))
+		while (*from && isspace((int)*from))
 		    from++;
 		continue;
 	    }
@@ -695,6 +696,25 @@ STATIC BOOL CNFSread_config(void) {
 		metabuff_update = METACYCBUFF_UPDATE;
 	    else
 		metabuff_update = update;
+	} else if (strncmp(ctab[ctab_i], "refreshinterval:", 16) == 0) {
+	    if (refreshintervalfound) {
+		syslog(L_ERROR, "%s: duplicate refreshinterval entries", LocalLogName);
+		DISPOSE(config);
+		DISPOSE(ctab);
+		return FALSE;
+	    }
+	    refreshintervalfound = TRUE;
+	    refresh = atoi(ctab[ctab_i] + 16);
+	    if (refresh < 0) {
+		syslog(L_ERROR, "%s: invalid refreshinterval", LocalLogName);
+		DISPOSE(config);
+		DISPOSE(ctab);
+		return FALSE;
+	    }
+	    if (refresh == 0)
+		refresh_interval = REFRESH_INTERVAL;
+	    else
+		refresh_interval = refresh;
 	} else {
 	    syslog(L_ERROR, "%s: Bogus metacycbuff config line '%s' ignored",
 		   LocalLogName, ctab[ctab_i]);
@@ -813,17 +833,17 @@ STATIC void CNFSmunmapbitfields(void) {
 }
 
 STATIC int CNFSArtMayBeHere(CYCBUFF *cycbuff, CYCBUFF_OFF_T offset, U_INT32_T cycnum) {
-    static int	count = 0;
-    CYCBUFF	*tmp;
+    STATIC time_t       lastupdate = 0;
+    CYCBUFF	        *tmp;
 
-    if (SMpreopen) {
-	if (++count % 1000 == 0) {	/* XXX 1K articles is just a guess */
+    if (SMpreopen && !SMopenmode) {
+	if ((time(NULL) - lastupdate) > refresh_interval) {	/* XXX Changed to refresh every 30sec - cmo*/
 	    for (tmp = cycbufftab; tmp != (CYCBUFF *)NULL; tmp = tmp->next) {
 		CNFSReadFreeAndCycle(tmp);
 	    }
+	    lastupdate = time(NULL);
 	} else if (cycnum == cycbuff->cyclenum + 1) {	/* rollover ? */
 	    CNFSReadFreeAndCycle(cycbuff);
-	    count = 0;
 	}
     }
     /*
@@ -873,7 +893,7 @@ BOOL cnfs_init(BOOL *selfexpire) {
 	if ((pagesize = sysconf(_SC_PAGESIZE)) < 0) {
 	    syslog(L_ERROR, "%s: sysconf(_SC_PAGESIZE) failed: %m", LocalLogName);
 	    SMseterror(SMERR_INTERNAL, "sysconf(_SC_PAGESIZE) failed");
-	    return NULL;
+	    return FALSE;
 	}
 #else
 	pagesize = 16384;
@@ -881,7 +901,7 @@ BOOL cnfs_init(BOOL *selfexpire) {
 	if ((pagesize > CNFS_HDR_PAGESIZE) || (CNFS_HDR_PAGESIZE % pagesize)) {
 	    syslog(L_ERROR, "%s: CNFS_HDR_PAGESIZE (%d) is not a multiple of pagesize (%d)", LocalLogName, CNFS_HDR_PAGESIZE, pagesize);
 	    SMseterror(SMERR_INTERNAL, "CNFS_HDR_PAGESIZE not multiple of pagesize");
-	    return NULL;
+	    return FALSE;
 	}
     }
     if (STORAGE_TOKEN_LENGTH < 16) {
@@ -913,14 +933,12 @@ BOOL cnfs_init(BOOL *selfexpire) {
     return TRUE;
 }
 
-TOKEN cnfs_store(const ARTHANDLE article, STORAGECLASS class) {
+TOKEN cnfs_store(const ARTHANDLE article, const STORAGECLASS class) {
     TOKEN               token;
-    char		*p;
     CYCBUFF		*cycbuff = NULL;
     METACYCBUFF		*metacycbuff = NULL;
     int			i;
-    static char		buf[1024], *bufp = buf;
-    int			chars = 0;
+    static char		buf[1024];
     char		*artcycbuffname;
     CYCBUFF_OFF_T	artoffset, middle;
     U_INT32_T		artcyclenum;
@@ -980,7 +998,7 @@ TOKEN cnfs_store(const ARTHANDLE article, STORAGECLASS class) {
 	cah.arrived = htonl(time(NULL));
     else
 	cah.arrived = htonl(article.arrived);
-    cah.class = htonl(class);
+    cah.class = class;
 
     if (CNFSseek(cycbuff->fd, artoffset, SEEK_SET) < 0) {
 	SMseterror(SMERR_INTERNAL, "CNFSseek() failed");
@@ -1028,7 +1046,7 @@ TOKEN cnfs_store(const ARTHANDLE article, STORAGECLASS class) {
     return CNFSMakeToken(artcycbuffname, artoffset, artcyclenum, class, article.token);
 }
 
-ARTHANDLE *cnfs_retrieve(const TOKEN token, RETRTYPE amount) {
+ARTHANDLE *cnfs_retrieve(const TOKEN token, const RETRTYPE amount) {
     char		cycbuffname[9];
     CYCBUFF_OFF_T	offset;
     U_INT32_T		cycnum;
@@ -1037,10 +1055,11 @@ ARTHANDLE *cnfs_retrieve(const TOKEN token, RETRTYPE amount) {
     CNFSARTHEADER	cah;
     PRIV_CNFS		*private;
     char		*p;
-    long		pagefudge;
+    long		pagefudge, blockfudge;
     CYCBUFF_OFF_T	mmapoffset;
     static TOKEN	ret_token;
     static BOOL		nomessage = FALSE;
+    CYCBUFF_OFF_T	middle, limit;
 
     if (token.type != TOKEN_CNFS) {
 	SMseterror(SMERR_INTERNAL, NULL);
@@ -1105,6 +1124,7 @@ ARTHANDLE *cnfs_retrieve(const TOKEN token, RETRTYPE amount) {
 	if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
         return NULL;
     }
+#ifdef OLD_CNFS
     if(cah.size == htonl(0x1234) && ntohl(cah.arrived) < time(NULL)-10*365*24*3600) {
 	oldCNFSARTHEADER cahh;
 	*(CNFSARTHEADER *)&cahh = cah;
@@ -1122,7 +1142,8 @@ ARTHANDLE *cnfs_retrieve(const TOKEN token, RETRTYPE amount) {
 	cah.class = 0;
 	offset += sizeof(oldCNFSARTHEADER)-sizeof(CNFSARTHEADER);
     }
-    if (cycbuff->free > cycbuff->len - CNFS_BLOCKSIZE - ntohl(cah.size) - 1) {
+#endif /* OLD_CNFS */
+    if (offset > cycbuff->len - CNFS_BLOCKSIZE - ntohl(cah.size) - 1) {
         if (!SMpreopen) {
 	    SMseterror(SMERR_UNDEFINED, "CNFSARTHEADER size overflow");
 	    syslog(L_ERROR, "%s: could not match article size token %s %s:0x%s:%ld: %ld",
@@ -1132,7 +1153,7 @@ ARTHANDLE *cnfs_retrieve(const TOKEN token, RETRTYPE amount) {
 	    return NULL;
 	}
 	CNFSReadFreeAndCycle(cycbuff);
-	if (cycbuff->free > cycbuff->len - CNFS_BLOCKSIZE - ntohl(cah.size) - 1) {
+	if (offset > cycbuff->len - CNFS_BLOCKSIZE - ntohl(cah.size) - 1) {
 	    SMseterror(SMERR_UNDEFINED, "CNFSARTHEADER size overflow");
 	    syslog(L_ERROR, "%s: could not match article size token %s %s:0x%s:%ld: %ld",
 		LocalLogName, TokenToText(token), cycbuffname, CNFSofft2hex(offset, FALSE), cycnum, ntohl(cah.size));
@@ -1140,6 +1161,27 @@ ARTHANDLE *cnfs_retrieve(const TOKEN token, RETRTYPE amount) {
 	    return NULL;
 	}
     }
+    /* check the bitmap to ensure cah.size is not broken */
+    /* cannot believe cycbuff->free, since it may not be updated by writer */
+    blockfudge = (sizeof(cah) + ntohl(cah.size)) % CNFS_BLOCKSIZE;
+    limit = offset + sizeof(cah) + ntohl(cah.size) - blockfudge + CNFS_BLOCKSIZE;
+    for (middle = offset + CNFS_BLOCKSIZE; middle < limit;
+	middle += CNFS_BLOCKSIZE) {
+	if (CNFSUsedBlock(cycbuff, middle, FALSE, FALSE) != 0)
+	/* Bitmap set.  This article assumes to be broken */
+	break;
+    }
+    if (middle != limit) {
+	char buf1[24], buf2[24], buf3[24];
+	strcpy(buf1, CNFSofft2hex(cycbuff->free, FALSE));
+	strcpy(buf2, CNFSofft2hex(middle, FALSE));
+	strcpy(buf3, CNFSofft2hex(limit, FALSE));
+	SMseterror(SMERR_UNDEFINED, "size overflows bitmaps");
+	syslog(L_ERROR, "%s: size overflows bitmaps %s %s:0x%s:0x%s:0x%s: %ld",
+	LocalLogName, TokenToText(token), cycbuffname, buf1, buf2, buf3, ntohl(cah.size));
+	DISPOSE(art);
+	return NULL;
+    }
     private = NEW(PRIV_CNFS, 1);
     art->private = (void *)private;
     art->arrived = ntohl(cah.arrived);
@@ -1158,8 +1200,12 @@ ARTHANDLE *cnfs_retrieve(const TOKEN token, RETRTYPE amount) {
 	    if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
 	    return NULL;
 	}
+#if defined(MADV_SEQUENTIAL) && defined(HAVE_MADVISE)
+	madvise(private->base, private->len, MADV_SEQUENTIAL);
+#endif
     } else {
 	private->base = NEW(char, ntohl(cah.size));
+	pagefudge = 0;
 	if (read(cycbuff->fd, private->base, ntohl(cah.size)) < 0) {
 	    SMseterror(SMERR_UNDEFINED, "read failed");
 	    syslog(L_ERROR, "%s: could not read token %s %s:0x%s:%ld: %m",
@@ -1182,7 +1228,7 @@ ARTHANDLE *cnfs_retrieve(const TOKEN token, RETRTYPE amount) {
     if ((p = SMFindBody(innconf->articlemmap ? private->base + pagefudge : private->base, art->len)) == NULL) {
         SMseterror(SMERR_NOBODY, NULL);
 	if (innconf->articlemmap) {
-#if defined(MADV_DONTNEED) && !defined(_nec_ews)
+#if defined(MADV_DONTNEED) && defined(HAVE_MADVISE)
 	    madvise(private->base, private->len, MADV_DONTNEED);
 #endif
 	    munmap(private->base, private->len);
@@ -1215,7 +1261,7 @@ ARTHANDLE *cnfs_retrieve(const TOKEN token, RETRTYPE amount) {
     }
     SMseterror(SMERR_UNDEFINED, "Invalid retrieve request");
     if (innconf->articlemmap) {
-#if defined(MADV_DONTNEED) && !defined(_nec_ews)
+#if defined(MADV_DONTNEED) && defined(HAVE_MADVISE)
 	madvise(private->base, private->len, MADV_DONTNEED);
 #endif
 	munmap(private->base, private->len);
@@ -1236,7 +1282,7 @@ void cnfs_freearticle(ARTHANDLE *article) {
     if (article->private) {
 	private = (PRIV_CNFS *)article->private;
 	if (innconf->articlemmap) {
-#if defined(MADV_DONTNEED) && !defined(_nec_ews)
+#if defined(MADV_DONTNEED) && defined(HAVE_MADVISE)
 	    madvise(private->base, private->len, MADV_DONTNEED);
 #endif	/* MADV_DONTNEED */
 	    munmap(private->base, private->len);
@@ -1288,14 +1334,14 @@ BOOL cnfs_cancel(TOKEN token) {
     return TRUE;
 }
 
-ARTHANDLE *cnfs_next(const ARTHANDLE *article, RETRTYPE amount) {
+ARTHANDLE *cnfs_next(const ARTHANDLE *article, const RETRTYPE amount) {
     ARTHANDLE           *art;
     CYCBUFF		*cycbuff;
     PRIV_CNFS		priv, *private;
-    CYCBUFF_OFF_T	middle;
+    CYCBUFF_OFF_T	middle, limit;
     CNFSARTHEADER	cah;
     CYCBUFF_OFF_T	offset;
-    long		pagefudge;
+    long		pagefudge, blockfudge;
     static TOKEN	token;
     int			tonextblock;
     CYCBUFF_OFF_T	mmapoffset;
@@ -1311,12 +1357,20 @@ ARTHANDLE *cnfs_next(const ARTHANDLE *article, RETRTYPE amount) {
 	DISPOSE(article->private);
 	DISPOSE(article);
 	if (innconf->articlemmap) {
-#if defined(MADV_DONTNEED) && !defined(_nec_ews)
+#if defined(MADV_DONTNEED) && defined(HAVE_MADVISE)
 	    madvise(priv.base, priv.len, MADV_DONTNEED);  
 #endif
 	    munmap(priv.base, priv.len);
-	} else
-	    DISPOSE(priv.base);
+	} else {
+	    /* In the case we return art->data = NULL, we
+             * must not free an already stale pointer.
+               -mibsoft@mibsoftware.com
+	     */
+	    if (priv.base) {
+	        DISPOSE(priv.base);
+		priv.base = 0;
+	    }
+	}
 	cycbuff = priv.cycbuff;
     }
 
@@ -1374,6 +1428,7 @@ ARTHANDLE *cnfs_next(const ARTHANDLE *article, RETRTYPE amount) {
 	if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
 	return (ARTHANDLE *)NULL;
     }
+#ifdef OLD_CNFS
     if(cah.size == htonl(0x1234) && ntohl(cah.arrived) < time(NULL)-10*365*24*3600) {
 	oldCNFSARTHEADER cahh;
 	*(CNFSARTHEADER *)&cahh = cah;
@@ -1387,6 +1442,7 @@ ARTHANDLE *cnfs_next(const ARTHANDLE *article, RETRTYPE amount) {
 	priv.offset += sizeof(oldCNFSARTHEADER)-sizeof(CNFSARTHEADER);
 	offset += sizeof(oldCNFSARTHEADER)-sizeof(CNFSARTHEADER);
     }
+#endif /* OLD_CNFS */
     art = NEW(ARTHANDLE, 1);
     private = NEW(PRIV_CNFS, 1);
     art->private = (void *)private;
@@ -1398,8 +1454,42 @@ ARTHANDLE *cnfs_next(const ARTHANDLE *article, RETRTYPE amount) {
 	private->offset += CNFS_BLOCKSIZE;
 	art->data = NULL;
 	art->len = 0;
+	if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
 	return art;
     }
+    /* check the bitmap to ensure cah.size is not broken */
+    blockfudge = (sizeof(cah) + ntohl(cah.size)) % CNFS_BLOCKSIZE;
+    limit = private->offset + sizeof(cah) + ntohl(cah.size) - blockfudge + CNFS_BLOCKSIZE;
+    if (offset < cycbuff->free) {
+	for (middle = offset + CNFS_BLOCKSIZE; (middle < cycbuff->free) && (middle < limit);
+	    middle += CNFS_BLOCKSIZE) {
+	    if (CNFSUsedBlock(cycbuff, middle, FALSE, FALSE) != 0)
+		/* Bitmap set.  This article assumes to be broken */
+		break;
+	}
+	if ((middle >= cycbuff->free) || (middle != limit)) {
+	    private->offset = middle;
+	    art->data = NULL;
+	    art->len = 0;
+	    if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
+	    return art;
+	}
+    } else {
+	for (middle = offset + CNFS_BLOCKSIZE; (middle < cycbuff->len) && (middle < limit);
+	    middle += CNFS_BLOCKSIZE) {
+	    if (CNFSUsedBlock(cycbuff, middle, FALSE, FALSE) != 0)
+		/* Bitmap set.  This article assumes to be broken */
+		break;
+	}
+	if ((middle >= cycbuff->len) || (middle != limit)) {
+	    private->offset = middle;
+	    art->data = NULL;
+	    art->len = 0;
+	    if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
+	    return art;
+	}
+    }
+
     private->offset += (CYCBUFF_OFF_T) ntohl(cah.size) + sizeof(cah);
     tonextblock = CNFS_BLOCKSIZE - (private->offset & (CNFS_BLOCKSIZE - 1));
     private->offset += (CYCBUFF_OFF_T) tonextblock;
@@ -1418,14 +1508,19 @@ ARTHANDLE *cnfs_next(const ARTHANDLE *article, RETRTYPE amount) {
 	    if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
 	    return art;
 	}
+#if defined(MADV_SEQUENTIAL) && defined(HAVE_MADVISE)
+	madvise(private->base, private->len, MADV_SEQUENTIAL);
+#endif
     } else {
 	private->base = NEW(char, ntohl(cah.size));
+	pagefudge = 0;
 	if (read(cycbuff->fd, private->base, ntohl(cah.size)) < 0) {
 	    art->data = NULL;
 	    art->len = 0;
 	    if (!SMpreopen) CNFSshutdowncycbuff(cycbuff);
-	    return art;
 	    DISPOSE(private->base);
+	    private->base = 0;
+	    return art;
 	}
     }
     art->len = ntohl(cah.size);
